@@ -17,10 +17,6 @@ namespace Algolist {
 /*************************************************
 * Lookup an algorithm in the table               *
 *************************************************/
-BlockCipher*               get_block_cipher(const std::string&);
-StreamCipher*              get_stream_cipher(const std::string&);
-HashFunction*              get_hash(const std::string&);
-MessageAuthenticationCode* get_mac(const std::string&);
 S2K*                       get_s2k(const std::string&);
 BlockCipherModePaddingMethod* get_bc_pad(const std::string&);
 
