@@ -4,10 +4,12 @@
 *************************************************/
 
 #include <botan/x509stor.h>
+#include <botan/parsing.h>
 #include <botan/pubkey.h>
 #include <botan/look_pk.h>
 #include <botan/oids.h>
 #include <botan/conf.h>
+#include <botan/util.h>
 #include <algorithm>
 #include <memory>
 
