@@ -8,6 +8,7 @@
 // PURPOSE.
 
 #include <iostream>
+#include <cstring>
 
 #include "cmd.hh"
 #include "diff_patch.hh"
@@ -26,123 +27,130 @@ using std::map;
 using std::set;
 using std::string;
 using std::vector;
+using std::strlen;
 
 using boost::shared_ptr;
 
-struct update_source
-  : public file_content_source
+static void
+three_way_merge(roster_t const & ancestor_roster,
+                roster_t const & left_roster, roster_t const & right_roster,
+                roster_merge_result & result)
 {
-  map<file_id, file_data> & temporary_store;
-  app_state & app;
-  update_source (map<file_id, file_data> & tmp,
-                 app_state & app)
-    : temporary_store(tmp), app(app)
-  {}
-  void get_file_content(file_id const & fid,
-                        file_data & dat) const
-  {
-    map<file_id, file_data>::const_iterator 
-      i = temporary_store.find(fid);
+  MM(ancestor_roster);
+  MM(left_roster);
+  MM(right_roster);
 
-    if (i != temporary_store.end())
-      dat = i->second;
-    else
-      app.db.get_file_version(fid, dat);
-  }
-};
+  // Make some fake rids
+  revision_id ancestor_rid(fake_id()); MM(ancestor_rid);
+  revision_id left_rid(fake_id()); MM(left_rid);
+  revision_id right_rid(fake_id()); MM(right_rid);
+  
+  // Mark up the ANCESTOR
+  marking_map ancestor_markings; MM(ancestor_markings);
+  mark_roster_with_no_parents(ancestor_rid, ancestor_roster, ancestor_markings);
 
+  // Mark up the LEFT roster
+  marking_map left_markings; MM(left_markings);
+  mark_roster_with_one_parent(ancestor_roster, ancestor_markings,
+                              left_rid, left_roster, left_markings);
+  
+  // Mark up the RIGHT roster
+  marking_map right_markings; MM(right_markings);
+  mark_roster_with_one_parent(ancestor_roster, ancestor_markings,
+                              right_rid, right_roster, right_markings);
+
+  // Make the synthetic graph, by creating uncommon ancestor sets
+  std::set<revision_id> left_uncommon_ancestors, right_uncommon_ancestors;
+  safe_insert(left_uncommon_ancestors, left_rid);
+  safe_insert(right_uncommon_ancestors, right_rid);
+
+  // And do the merge
+  roster_merge(left_roster, left_markings, left_uncommon_ancestors,
+               right_roster, right_markings, right_uncommon_ancestors,
+               result);
+}
+  
 CMD(update, N_("workspace"), "",
     N_("update workspace.\n"
        "This command modifies your workspace to be based off of a\n"
        "different revision, preserving uncommitted changes as it does so.\n"
        "If a revision is given, update the workspace to that revision.\n"
        "If not, update the workspace to the head of the branch."),
-    OPT_BRANCH_NAME % OPT_REVISION)
+    options::opts::branch | options::opts::revision)
 {
-  revision_t r_working;
-  roster_t working_roster, chosen_roster, target_roster;
-  shared_ptr<roster_t> old_roster = shared_ptr<roster_t>(new roster_t());
-  marking_map working_mm, chosen_mm, merged_mm, target_mm;
-  revision_id r_old_id, r_working_id, r_chosen_id, r_target_id;
-  temp_node_id_source nis;
-
   if (args.size() > 0)
     throw usage(name);
 
-  if (app.revision_selectors.size() > 1)
+  if (app.opts.revision_selectors.size() > 1)
     throw usage(name);
 
   app.require_workspace();
 
-  // FIXME: the next few lines are a little bit expensive insofar as they
-  // load the base roster twice. The API could use some factoring or
-  // such. But it should work for now; revisit if performance is
-  // intolerable.
+  // Figure out where we are
 
-  get_base_and_current_roster_shape(*old_roster, 
-                                    working_roster, nis, app);
-  update_current_roster_from_filesystem(working_roster, app);
+  revision_id old_rid;
+  app.work.get_revision_id(old_rid);
 
-  get_revision_id(r_old_id);
-  make_revision(r_old_id, *old_roster, working_roster, r_working);
-
-  calculate_ident(r_working, r_working_id);
-  I(r_working.edges.size() == 1);
-  r_old_id = edge_old_revision(r_working.edges.begin());
-  make_roster_for_base_plus_cset(r_old_id,
-                                 edge_changes(r_working.edges.begin()),
-                                 r_working_id,
-                                 working_roster, working_mm, nis, app);
-
-  N(!null_id(r_old_id),
+  N(!null_id(old_rid),
     F("this workspace is a new project; cannot update"));
 
-  if (app.revision_selectors.size() == 0)
+  // Figure out where we're going
+
+  revision_id chosen_rid;
+  if (app.opts.revision_selectors.size() == 0)
     {
-      P(F("updating along branch '%s'") % app.branch_name);
+      P(F("updating along branch '%s'") % app.opts.branch_name);
       set<revision_id> candidates;
-      pick_update_candidates(r_old_id, app, candidates);
+      pick_update_candidates(old_rid, app, candidates);
       N(!candidates.empty(),
         F("your request matches no descendents of the current revision\n"
           "in fact, it doesn't even match the current revision\n"
-          "maybe you want --revision=<rev on other branch>"));
+          "maybe you want something like --revision=h:%s")
+        % app.opts.branch_name);
       if (candidates.size() != 1)
         {
           P(F("multiple update candidates:"));
           for (set<revision_id>::const_iterator i = candidates.begin();
                i != candidates.end(); ++i)
             P(i18n_format("  %s") % describe_revision(app, *i));
-          P(F("choose one with '%s update -r<id>'") % app.prog_name);
+          P(F("choose one with '%s update -r<id>'") % ui.prog_name);
           E(false, F("multiple update candidates remain after selection"));
         }
-      r_chosen_id = *(candidates.begin());
+      chosen_rid = *(candidates.begin());
     }
   else
     {
-      complete(app, app.revision_selectors[0](), r_chosen_id);
-      N(app.db.revision_exists(r_chosen_id),
-        F("no such revision '%s'") % r_chosen_id);
+      complete(app, app.opts.revision_selectors[0](), chosen_rid);
+      N(app.db.revision_exists(chosen_rid),
+        F("no such revision '%s'") % chosen_rid);
     }
+  I(!null_id(chosen_rid));
 
+  // do this notification before checking to see if we can bail out early,
+  // because when you are at one of several heads, and you hit update, you
+  // want to know that merging would let you update further.
   notify_if_multiple_heads(app);
 
-  if (r_old_id == r_chosen_id)
+  if (old_rid == chosen_rid)
     {
-      P(F("already up to date at %s") % r_old_id);
+      P(F("already up to date at %s") % old_rid);
       // do still switch the workspace branch, in case they have used
       // update to switch branches.
-      if (!app.branch_name().empty())
+      if (!app.opts.branch_name().empty())
         app.make_branch_sticky();
       return;
     }
 
-  P(F("selected update target %s") % r_chosen_id);
+  P(F("selected update target %s") % chosen_rid);
 
+  // Fiddle around with branches, in an attempt to guess what the user
+  // wants.
+  
   bool switched_branch = false;
   {
     // figure out which branches the target is in
     vector< revision<cert> > certs;
-    app.db.get_revision_certs(r_chosen_id, branch_cert_name, certs);
+    app.db.get_revision_certs(chosen_rid, branch_cert_name, certs);
     erase_bogus_certs(certs, app);
 
     set< utf8 > branches;
@@ -154,9 +162,9 @@ CMD(update, N_("workspace"), "",
         branches.insert(utf8(b()));
       }
 
-    if (branches.find(app.branch_name) != branches.end())
+    if (branches.find(app.opts.branch_name) != branches.end())
       {
-        L(FL("using existing branch %s") % app.branch_name());
+        L(FL("using existing branch %s") % app.opts.branch_name());
       }
     else
       {
@@ -174,124 +182,88 @@ CMD(update, N_("workspace"), "",
         else if (branches.size() == 1)
           {
             // one non-matching, inform and update
-            app.branch_name = (*(branches.begin()))();
+            app.opts.branch_name = (*(branches.begin()))();
             switched_branch = true;
-            P(F("switching to branch %s") % app.branch_name());
+            P(F("switching to branch %s") % app.opts.branch_name());
           }
         else
           {
             I(branches.size() == 0);
             W(F("target revision not in any branch\n"
                 "next commit will use branch %s")
-              % app.branch_name());
+              % app.opts.branch_name());
           }
       }
   }
 
-  app.db.get_roster(r_chosen_id, chosen_roster, chosen_mm);
 
-  set<revision_id>
-    working_uncommon_ancestors,
-    chosen_uncommon_ancestors;
+  // Okay, we have a target, we have a branch, let's do this merge!
 
-  if (is_ancestor(r_old_id, r_chosen_id, app))
-    {
-      target_roster = chosen_roster;
-      target_mm = chosen_mm;
-      r_target_id = r_chosen_id;
-      app.db.get_uncommon_ancestors(r_old_id, r_chosen_id,
-                                    working_uncommon_ancestors,
-                                    chosen_uncommon_ancestors);
-    }
-  else
-    {
-      cset transplant;
-      make_cset (*old_roster, chosen_roster, transplant);
-
-      // Just pick some unused revid, all that's important is that it not
-      // match the work revision or any ancestors of the base revision.
-      r_target_id = revision_id(hexenc<id>("5432100000000000000000000500000000000000"));
-      make_roster_for_base_plus_cset(r_old_id,
-                                     transplant,
-                                     r_target_id,
-                                     target_roster, target_mm, nis, app);
-      chosen_uncommon_ancestors.insert(r_target_id);
-    }
-
-  // Note that under the definition of mark-merge, the workspace is an
-  // "uncommon ancestor" of itself too, even though it was not present in
-  // the database (hence not returned by the query above).
-
-  working_uncommon_ancestors.insert(r_working_id);
-
-  // Now merge the working roster with the chosen target.
-
-  roster_merge_result result;
-  roster_merge(working_roster, working_mm, working_uncommon_ancestors,
-               target_roster, target_mm, chosen_uncommon_ancestors,
-               result);
-
-  roster_t & merged_roster = result.roster;
-
-  content_merge_workspace_adaptor wca(app, old_roster);
-  resolve_merge_conflicts (r_old_id, r_target_id,
-                           working_roster, target_roster,
-                           working_mm, target_mm,
-                           result, wca, app);
-
-  I(result.is_clean());
-
-  // Temporary node ids may appear if updating to a non-ancestor.
-  merged_roster.check_sane(true);
-
-  // We have the following
+  // We have:
   //
-  // old --> working
-  //   |         |
-  //   V         V
+  //    old  --> working
+  //     |         |
+  //     V         V
   //  chosen --> merged
   //
   // - old is the revision specified in _MTN/revision
   // - working is based on old and includes the workspace's changes
   // - chosen is the revision we're updating to and will end up in _MTN/revision
-  // - merged is the merge of working and chosen
+  // - merged is the merge of working and chosen, that will become the new
+  //   workspace
   //
   // we apply the working to merged cset to the workspace
   // and write the cset from chosen to merged changeset in _MTN/work
+  
+  temp_node_id_source nis;
 
-  cset update, remaining;
+  // Get the OLD and WORKING rosters
+  shared_ptr<roster_t> old_roster = shared_ptr<roster_t>(new roster_t());
+  MM(*old_roster);
+  roster_t working_roster; MM(working_roster);
+  app.work.get_base_and_current_roster_shape(*old_roster, working_roster, nis);
+  app.work.update_current_roster_from_filesystem(working_roster);
+
+  // Get the CHOSEN roster
+  roster_t chosen_roster; MM(chosen_roster);
+  app.db.get_roster(chosen_rid, chosen_roster);
+  
+  // And finally do the merge
+  roster_merge_result result;
+  three_way_merge(*old_roster, working_roster, chosen_roster, result);
+
+  roster_t & merged_roster = result.roster;
+
+  content_merge_workspace_adaptor wca(app, old_roster);
+  resolve_merge_conflicts(working_roster, chosen_roster,
+                          result, wca, app);
+
+  // Make sure it worked...
+  I(result.is_clean());
+  merged_roster.check_sane(true);
+
+  // Now finally modify the workspace
+  cset update;
   make_cset(working_roster, merged_roster, update);
-  make_cset(target_roster, merged_roster, remaining);
+  app.work.perform_content_update(update, wca);
 
-  //   {
-  //     data t1, t2, t3;
-  //     write_cset(update, t1);
-  //     write_cset(remaining, t2);
-  //     write_manifest_of_roster(merged_roster, t3);
-  //     P(F("updating workspace with [[[\n%s\n]]]") % t1);
-  //     P(F("leaving residual work [[[\n%s\n]]]") % t2);
-  //     P(F("merged roster [[[\n%s\n]]]") % t3);
-  //   }
-
-  update_source fsource(wca.temporary_store, app);
-  editable_working_tree ewt(app, fsource);
-  update.apply_to(ewt);
+  revision_t remaining;
+  make_revision_for_workspace(chosen_rid, chosen_roster,
+                              merged_roster, remaining);
 
   // small race condition here...
-  // nb: we write out r_chosen, not r_new, because the revision-on-disk
-  // is the basis of the workspace, not the workspace itself.
-  put_revision_id(r_chosen_id);
-  if (!app.branch_name().empty())
+  app.work.put_work_rev(remaining);
+
+  if (!app.opts.branch_name().empty())
     {
       app.make_branch_sticky();
     }
   if (switched_branch)
-    P(F("switched branch; next commit will use branch %s") % app.branch_name());
-  P(F("updated to base revision %s") % r_chosen_id);
+    P(F("switched branch; next commit will use branch %s") % app.opts.branch_name());
+  P(F("updated to base revision %s") % chosen_rid);
 
-  put_work_cset(remaining);
-  update_any_attrs(app);
-  maybe_update_inodeprints(app);
+  app.work.update_any_attrs();
+  app.work.maybe_update_inodeprints();
 }
 
 // Subroutine of CMD(merge) and CMD(explicit_merge).  Merge LEFT with RIGHT,
@@ -315,14 +287,14 @@ merge_two(revision_id const & left, revision_id const & right,
   ostringstream log;
   size_t fieldwidth = max(caller.size() + strlen(" of '"), strlen("and '"));
 
-  if (branch != app.branch_name())
+  if (branch != app.opts.branch_name())
     fieldwidth = max(fieldwidth, strlen("to branch '"));
 
   log << setw(fieldwidth - strlen(" of '")) << caller << " of '" << left
       << "'\n" << setw(fieldwidth) << "and '" << right
       << "'\n";
 
-  if (branch != app.branch_name())
+  if (branch != app.opts.branch_name())
     log << setw(fieldwidth) << "to branch '" << branch << "'\n";
 
   // Now it's time for the real work.
@@ -346,7 +318,7 @@ merge_two(revision_id const & left, revision_id const & right,
 // (Possibility: append the --message/--message-file text to the synthetic
 // log message constructed in merge_two().)
 CMD(merge, N_("tree"), "", N_("merge unmerged heads of branch"),
-    OPT_BRANCH_NAME % OPT_DATE % OPT_AUTHOR)
+    options::opts::branch | options::opts::date | options::opts::author)
 {
   typedef std::pair<revision_id, revision_id> revpair;
   typedef set<revision_id>::const_iterator rid_set_iter;
@@ -354,21 +326,21 @@ CMD(merge, N_("tree"), "", N_("merge unmerged heads of branch"),
   if (args.size() != 0)
     throw usage(name);
 
-  N(app.branch_name() != "",
+  N(app.opts.branch_name() != "",
     F("please specify a branch, with --branch=BRANCH"));
 
   set<revision_id> heads;
-  get_branch_heads(app.branch_name(), app, heads);
+  get_branch_heads(app.opts.branch_name(), app, heads);
 
-  N(heads.size() != 0, F("branch '%s' is empty") % app.branch_name);
+  N(heads.size() != 0, F("branch '%s' is empty") % app.opts.branch_name);
   if (heads.size() == 1)
     {
-      P(F("branch '%s' is already merged") % app.branch_name);
+      P(F("branch '%s' is already merged") % app.opts.branch_name);
       return;
     }
 
   P(FP("%d head on branch '%s'", "%d heads on branch '%s'", heads.size())
-    % heads.size() % app.branch_name);
+    % heads.size() % app.opts.branch_name);
 
   map<revision_id, revpair> heads_for_ancestor;
   set<revision_id> ancestors;
@@ -422,11 +394,11 @@ CMD(merge, N_("tree"), "", N_("merge unmerged heads of branch"),
       // corresponding pair of heads.
       revpair p = heads_for_ancestor[*ancestors.begin()];
       
-      merge_two(p.first, p.second, app.branch_name(), string("merge"), app);
+      merge_two(p.first, p.second, app.opts.branch_name(), string("merge"), app);
 
       ancestors.clear();
       heads_for_ancestor.clear();
-      get_branch_heads(app.branch_name(), app, heads);
+      get_branch_heads(app.opts.branch_name(), app, heads);
       pass++;
     }
 
@@ -440,13 +412,13 @@ CMD(merge, N_("tree"), "", N_("merge unmerged heads of branch"),
   revision_id right = *i++;
   I(i == heads.end());
   
-  merge_two(left, right, app.branch_name(), string("merge"), app);
+  merge_two(left, right, app.opts.branch_name(), string("merge"), app);
   P(F("note: your workspaces have not been updated"));
 }
 
 CMD(propagate, N_("tree"), N_("SOURCE-BRANCH DEST-BRANCH"),
     N_("merge from one branch to another asymmetrically"),
-    OPT_DATE % OPT_AUTHOR % OPT_MESSAGE % OPT_MSGFILE)
+    options::opts::date | options::opts::author | options::opts::message | options::opts::msgfile)
 {
   if (args.size() != 2)
     throw usage(name);
@@ -457,7 +429,7 @@ CMD(propagate, N_("tree"), N_("SOURCE-BRANCH DEST-BRANCH"),
 
 CMD(merge_into_dir, N_("tree"), N_("SOURCE-BRANCH DEST-BRANCH DIR"),
     N_("merge one branch into a subdirectory in another branch"),
-    OPT_DATE % OPT_AUTHOR % OPT_MESSAGE % OPT_MSGFILE)
+    options::opts::date | options::opts::author | options::opts::message | options::opts::msgfile)
 {
   //   This is a special merge operator, but very useful for people
   //   maintaining "slightly disparate but related" trees. It does a one-way
@@ -578,10 +550,8 @@ CMD(merge_into_dir, N_("tree"), N_("SOURCE-BRANCH DEST-BRANCH DIR"),
         content_merge_database_adaptor 
           dba(app, left_rid, right_rid, left_marking_map);
 
-        resolve_merge_conflicts (left_rid, right_rid,
-                                 left_roster, right_roster,
-                                 left_marking_map, right_marking_map,
-                                 result, dba, app);
+        resolve_merge_conflicts(left_roster, right_roster,
+                                result, dba, app);
 
         {
           dir_t moved_root = left_roster.root();
@@ -600,7 +570,7 @@ CMD(merge_into_dir, N_("tree"), N_("SOURCE-BRANCH DEST-BRANCH DIR"),
       cert_revision_in_branch(merged, idx(args, 1)(), app, dbw);
 
       bool log_message_given;
-      string log_message;
+      utf8 log_message;
       process_commit_message_args(log_message_given, log_message, app);
       if (!log_message_given)
         log_message = (FL("propagate from branch '%s' (head %s)\n"
@@ -619,7 +589,7 @@ CMD(explicit_merge, N_("tree"),
     N_("LEFT-REVISION RIGHT-REVISION DEST-BRANCH"),
     N_("merge two explicitly given revisions, "
        "placing result in given branch"),
-    OPT_DATE % OPT_AUTHOR)
+    options::opts::date | options::opts::author)
 {
   revision_id left, right;
   string branch;
@@ -644,7 +614,7 @@ CMD(explicit_merge, N_("tree"),
 CMD(show_conflicts, N_("informative"), N_("REV REV"), 
     N_("Show what conflicts would need to be resolved "
        "to merge the given revisions."),
-    OPT_BRANCH_NAME % OPT_DATE % OPT_AUTHOR)
+    options::opts::branch | options::opts::date | options::opts::author)
 {
   if (args.size() != 2)
     throw usage(name);
@@ -694,15 +664,15 @@ CMD(pluck, N_("workspace"), N_("[-r FROM] -r TO [PATH...]"),
        "compared to its parent.\n"                                                                                  
        "\n"
        "If two revisions are given, applies the changes made to get from the\n"  
-       "first revision to the second."),                                                                            
-    OPT_REVISION % OPT_DEPTH % OPT_EXCLUDE)
+       "first revision to the second."),
+    options::opts::revision | options::opts::depth | options::opts::exclude)
 {
   // Work out our arguments
   revision_id from_rid, to_rid;
 
-  if (app.revision_selectors.size() == 1)
+  if (app.opts.revision_selectors.size() == 1)
     {
-      complete(app, idx(app.revision_selectors, 0)(), to_rid);
+      complete(app, idx(app.opts.revision_selectors, 0)(), to_rid);
       N(app.db.revision_exists(to_rid),
         F("no such revision '%s'") % to_rid);
       std::set<revision_id> parents;
@@ -712,15 +682,15 @@ CMD(pluck, N_("workspace"), N_("[-r FROM] -r TO [PATH...]"),
           "to apply the changes relative to one of its parents, use:\n"
           "  %s pluck -r PARENT -r %s")
         % to_rid
-        % app.prog_name % to_rid);
+        % ui.prog_name % to_rid);
       from_rid = *parents.begin();
     }
-  else if (app.revision_selectors.size() == 2)
+  else if (app.opts.revision_selectors.size() == 2)
     {
-      complete(app, idx(app.revision_selectors, 0)(), from_rid);
+      complete(app, idx(app.opts.revision_selectors, 0)(), from_rid);
       N(app.db.revision_exists(from_rid),
         F("no such revision '%s'") % from_rid);
-      complete(app, idx(app.revision_selectors, 1)(), to_rid);
+      complete(app, idx(app.opts.revision_selectors, 1)(), to_rid);
       N(app.db.revision_exists(to_rid),
         F("no such revision '%s'") % to_rid);
     }
@@ -768,61 +738,41 @@ CMD(pluck, N_("workspace"), N_("[-r FROM] -r TO [PATH...]"),
   // Get the WORKING roster, and also the base roster while we're at it
   roster_t working_roster; MM(working_roster);
   roster_t base_roster; MM(base_roster);
-  get_base_and_current_roster_shape(base_roster, working_roster,
-                                    nis, app);
-  update_current_roster_from_filesystem(working_roster, app);
+  app.work.get_base_and_current_roster_shape(base_roster, working_roster, nis);
+  app.work.update_current_roster_from_filesystem(working_roster);
 
-  // Get the FROM->TO cset
+  // Get the FROM->TO cset...
   cset from_to_to; MM(from_to_to);
   cset from_to_to_excluded; MM(from_to_to_excluded);
   {
     roster_t to_true_roster;
     app.db.get_roster(to_rid, to_true_roster);
     node_restriction mask(args_to_paths(args),
-                          args_to_paths(app.exclude_patterns),
+                          args_to_paths(app.opts.exclude_patterns),
+                          app.opts.depth,
                           *from_roster, to_true_roster, app);
     make_restricted_csets(*from_roster, to_true_roster,
                           from_to_to, from_to_to_excluded,
                           mask);
+    check_restricted_cset(*from_roster, from_to_to);
   }
-
-  // Use a fake rid
-  revision_id working_rid(std::string("0000000000000000000000000000000000000001"));
-
-  // Mark up the FROM roster
-  marking_map from_markings; MM(from_markings);
-  mark_roster_with_no_parents(from_rid, *from_roster, from_markings);
-
-  // Mark up the WORKING roster
-  marking_map working_markings; MM(working_markings);
-  mark_roster_with_one_parent(*from_roster, from_markings,
-                              working_rid, working_roster,
-                              working_markings);
-  
-  // Create and mark up the TO roster
+  N(!from_to_to.empty(), F("no changes to be applied"));
+  // ...and use it to create the TO roster
   roster_t to_roster; MM(to_roster);
-  marking_map to_markings; MM(to_markings);
-  make_roster_for_base_plus_cset(from_rid, from_to_to, to_rid,
-                                 to_roster, to_markings, nis,
-                                 app);
-
-  // Set up the synthetic graph, by creating uncommon ancestor sets
-  std::set<revision_id> working_uncommon_ancestors, to_uncommon_ancestors;
-  safe_insert(working_uncommon_ancestors, working_rid);
-  safe_insert(to_uncommon_ancestors, to_rid);
+  {
+    to_roster = *from_roster;
+    editable_roster_base editable_to_roster(to_roster, nis);
+    from_to_to.apply_to(editable_to_roster);
+  }
 
   // Now do the merge
   roster_merge_result result;
-  roster_merge(working_roster, working_markings, working_uncommon_ancestors,
-               to_roster, to_markings, to_uncommon_ancestors,
-               result);
+  three_way_merge(*from_roster, working_roster, to_roster, result);
 
   roster_t & merged_roster = result.roster;
 
   content_merge_workspace_adaptor wca(app, from_roster);
-  resolve_merge_conflicts(working_rid, to_rid,
-                          working_roster, to_roster,
-                          working_markings, to_markings,
+  resolve_merge_conflicts(working_roster, to_roster,
                           result, wca, app);
 
   I(result.is_clean());
@@ -830,27 +780,29 @@ CMD(pluck, N_("workspace"), N_("[-r FROM] -r TO [PATH...]"),
   merged_roster.check_sane(true);
 
   // we apply the working to merged cset to the workspace 
-  // and write the cset from the base to merged roster in _MTN/work
-  cset update, remaining;
+  cset update;
   MM(update);
-  MM(remaining);
   make_cset(working_roster, merged_roster, update);
-  make_cset(base_roster, merged_roster, remaining);
+  E(!update.empty(), F("no changes were applied"));
+  app.work.perform_content_update(update, wca);
 
-  update_source fsource(wca.temporary_store, app);
-  editable_working_tree ewt(app, fsource);
-  update.apply_to(ewt);
-  
-  // small race condition here...
   P(F("applied changes to workspace"));
 
-  put_work_cset(remaining);
-  update_any_attrs(app);
+  // and record any remaining changes in _MTN/revision
+  revision_id base_id;
+  revision_t remaining;
+  MM(remaining);
+  app.work.get_revision_id(base_id);
+  make_revision_for_workspace(base_id, base_roster, merged_roster, remaining);
+
+  // small race condition here...
+  app.work.put_work_rev(remaining);
+  app.work.update_any_attrs();
   
   // add a note to the user log file about what we did
   {
-    data log;
-    read_user_log(log);
+    utf8 log;
+    app.work.read_user_log(log);
     std::string log_str = log();
     if (!log_str.empty())
       log_str += "\n";
@@ -862,28 +814,28 @@ CMD(pluck, N_("workspace"), N_("[-r FROM] -r TO [PATH...]"),
       log_str += (FL("applied partial changes from %s\n"
                      "                     through %s\n")
                   % from_rid % to_rid).str();
-    write_user_log(data(log_str));
+    app.work.write_user_log(utf8(log_str));
   }
 }
 
 CMD(heads, N_("tree"), "", N_("show unmerged head revisions of branch"),
-    OPT_BRANCH_NAME)
+    options::opts::branch)
 {
   set<revision_id> heads;
   if (args.size() != 0)
     throw usage(name);
 
-  N(app.branch_name() != "",
+  N(app.opts.branch_name() != "",
     F("please specify a branch, with --branch=BRANCH"));
 
-  get_branch_heads(app.branch_name(), app, heads);
+  get_branch_heads(app.opts.branch_name(), app, heads);
 
   if (heads.size() == 0)
-    P(F("branch '%s' is empty") % app.branch_name);
+    P(F("branch '%s' is empty") % app.opts.branch_name);
   else if (heads.size() == 1)
-    P(F("branch '%s' is currently merged:") % app.branch_name);
+    P(F("branch '%s' is currently merged:") % app.opts.branch_name);
   else
-    P(F("branch '%s' is currently unmerged:") % app.branch_name);
+    P(F("branch '%s' is currently unmerged:") % app.opts.branch_name);
 
   for (set<revision_id>::const_iterator i = heads.begin();
        i != heads.end(); ++i)
@@ -892,11 +844,11 @@ CMD(heads, N_("tree"), "", N_("show unmerged head revisions of branch"),
 
 CMD(get_roster, N_("debug"), N_("REVID"),
     N_("dump the roster associated with the given REVID"),
-    OPT_NONE)
+    options::opts::none)
 {
   revision_id rid;
   if (args.size() == 0)
-    get_revision_id(rid);
+    app.work.get_revision_id(rid);
   else if (args.size() == 1)
     complete(app, idx(args, 0)(), rid);
   else
