@@ -16,7 +16,7 @@ CMD(rcs_import, N_("debug"), N_("RCSFILE..."),
     N_("parse versions in RCS files\n"
        "this command doesn't reconstruct or import revisions."
        "you probably want cvs_import"),
-    OPT_BRANCH_NAME)
+    options::opts::branch)
 {
   if (args.size() < 1)
     throw usage(name);
@@ -31,7 +31,7 @@ CMD(rcs_import, N_("debug"), N_("RCSFILE..."),
 
 CMD(cvs_import, N_("rcs"), N_("CVSROOT"), 
     N_("import all versions in CVS repository"),
-    OPT_BRANCH_NAME)
+    options::opts::branch)
 {
   if (args.size() != 1)
     throw usage(name);
