@@ -18,6 +18,7 @@
 #include "revision.hh"
 #include "transforms.hh"
 #include "work.hh"
+#include "charset.hh"
 
 using std::cout;
 using std::make_pair;
@@ -32,50 +33,63 @@ using boost::shared_ptr;
 static void
 get_log_message_interactively(revision_t const & cs,
                               app_state & app,
-                              string & log_message)
+                              utf8 & log_message)
 {
-  string commentary;
-  data summary, user_log_message;
+  revision_data summary;
   write_revision(cs, summary);
-  read_user_log(user_log_message);
-  commentary += string(70, '-') + "\n";
-  commentary += _("Enter a description of this change.\n"
-                  "Lines beginning with `MTN:' "
-                  "are removed automatically.\n");
-  commentary += "\n";
-  commentary += summary();
-  commentary += string(70, '-') + "\n";
+  external summary_external;
+  utf8_to_system(utf8(summary.inner()()), summary_external);
 
-  N(app.lua.hook_edit_comment(commentary, user_log_message(), log_message),
+  string commentary_str;
+  commentary_str += string(70, '-') + "\n";
+  commentary_str += _("Enter a description of this change.\n"
+                      "Lines beginning with `MTN:' "
+                      "are removed automatically.");
+  commentary_str += "\n\n";
+  commentary_str += summary_external();
+  commentary_str += string(70, '-') + "\n";
+
+  external commentary(commentary_str);
+
+  utf8 user_log_message;
+  app.work.read_user_log(user_log_message);
+  external user_log_message_external;
+  utf8_to_system(user_log_message, user_log_message_external);
+
+  external log_message_external;
+  N(app.lua.hook_edit_comment(commentary, user_log_message_external,
+                              log_message_external),
     F("edit of log message failed"));
+  system_to_utf8(log_message_external, log_message);
 }
 
 CMD(revert, N_("workspace"), N_("[PATH]..."),
     N_("revert file(s), dir(s) or entire workspace (\".\")"),
-    OPT_DEPTH % OPT_EXCLUDE % OPT_MISSING)
+    options::opts::depth | options::opts::exclude | options::opts::missing)
 {
   temp_node_id_source nis;
   roster_t old_roster, new_roster;
   cset included, excluded;
 
-  N(app.missing || !args.empty() || !app.exclude_patterns.empty(),
+  N(app.opts.missing || !args.empty() || !app.opts.exclude_patterns.empty(),
     F("you must pass at least one path to 'revert' (perhaps '.')"));
 
   app.require_workspace();
 
-  get_base_and_current_roster_shape(old_roster, new_roster, nis, app);
+  app.work.get_base_and_current_roster_shape(old_roster, new_roster, nis);
 
-  node_restriction mask(args_to_paths(args), args_to_paths(app.exclude_patterns),
+  node_restriction mask(args_to_paths(args), args_to_paths(app.opts.exclude_patterns),
+                        app.opts.depth,
                         old_roster, new_roster, app);
 
-  if (app.missing)
+  if (app.opts.missing)
     {
       // --missing is a further filter on the files included by a
       // restriction we first find all missing files included by the
       // specified args and then make a restriction that includes only
       // these missing files.
       path_set missing;
-      find_missing(new_roster, mask, missing);
+      app.work.find_missing(new_roster, mask, missing);
       if (missing.empty())
         {
           P(F("no missing files to revert"));
@@ -91,6 +105,7 @@ CMD(revert, N_("workspace"), N_("[PATH]..."),
         }
       // replace the original mask with a more restricted one
       mask = node_restriction(missing_files, std::vector<file_path>(),
+                              app.opts.depth,
                               old_roster, new_roster, app);
     }
 
@@ -104,7 +119,7 @@ CMD(revert, N_("workspace"), N_("[PATH]..."),
   check_restricted_cset(old_roster, excluded);
 
   node_map const & nodes = old_roster.all_nodes();
-  for (node_map::const_iterator i = nodes.begin(); 
+  for (node_map::const_iterator i = nodes.begin();
        i != nodes.end(); ++i)
     {
       node_id nid = i->first;
@@ -160,19 +175,27 @@ CMD(revert, N_("workspace"), N_("[PATH]..."),
   // been rewritten above but this may leave rename targets laying
   // around.
 
+  revision_t remaining;
+  revision_id base;
+  app.work.get_revision_id(base);
+  make_revision_for_workspace(base, excluded, remaining);
+
   // Race.
-  put_work_cset(excluded);
-  update_any_attrs(app);
-  maybe_update_inodeprints(app);
+  app.work.put_work_rev(remaining);
+  app.work.update_any_attrs();
+  app.work.maybe_update_inodeprints();
 }
 
 CMD(disapprove, N_("review"), N_("REVISION"),
     N_("disapprove of a particular revision"),
-    OPT_BRANCH_NAME)
+    options::opts::branch | options::opts::messages | options::opts::date |
+    options::opts::author)
 {
   if (args.size() != 1)
     throw usage(name);
 
+  utf8 log_message("");
+  bool log_message_given;
   revision_id r;
   revision_t rev, rev_inverse;
   shared_ptr<cset> cs_inverse(new cset());
@@ -180,11 +203,14 @@ CMD(disapprove, N_("review"), N_("REVISION"),
   app.db.get_revision(r, rev);
 
   N(rev.edges.size() == 1,
-    F("revision '%s' has %d changesets, cannot invert\n") % r % rev.edges.size());
+    F("revision %s has %d changesets, cannot invert") % r % rev.edges.size());
 
   cert_value branchname;
   guess_branch(r, app, branchname);
-  N(app.branch_name() != "", F("need --branch argument for disapproval"));
+  N(app.opts.branch_name() != "", F("need --branch argument for disapproval"));
+
+  process_commit_message_args(log_message_given, log_message, app,
+                              (FL("disapproval of revision '%s'") % r).str());
 
   edge_entry const & old_edge (*rev.edges.begin());
   app.db.get_revision_manifest(edge_old_revision(old_edge),
@@ -209,33 +235,50 @@ CMD(disapprove, N_("review"), N_("REVISION"),
     dbw.consume_revision_data(inv_id, rdat);
 
     cert_revision_in_branch(inv_id, branchname, app, dbw);
-    cert_revision_date_now(inv_id, app, dbw);
-    cert_revision_author_default(inv_id, app, dbw);
-    cert_revision_changelog(inv_id, 
-                            (FL("disapproval of revision '%s'") 
-                             % r).str(), app, dbw);
+    if (app.opts.date_given)
+      cert_revision_date_time(inv_id, app.opts.date, app, dbw);
+    else
+      cert_revision_date_now(inv_id, app, dbw);
+    if (app.opts.author().length() > 0)
+      cert_revision_author(inv_id, app.opts.author(), app, dbw);
+    else
+      cert_revision_author_default(inv_id, app, dbw);
+    cert_revision_changelog(inv_id, log_message, app, dbw);
     guard.commit();
   }
 }
 
 
 CMD(add, N_("workspace"), N_("[PATH]..."),
-    N_("add files to workspace"), OPT_UNKNOWN)
+    N_("add files to workspace"),
+    options::opts::unknown | options::opts::no_ignore |
+    options::opts::recursive)
 {
-  if (!app.unknown && (args.size() < 1))
+  if (!app.opts.unknown && (args.size() < 1))
     throw usage(name);
+  N(!app.opts.unknown || !app.opts.recursive,
+    F("cannot set '--unknown' and '--recursive' at the same time"));
+  N(!app.opts.unknown || !app.opts.no_ignore,
+    F("cannot set '--unknown' and '--no-respect-ignore' at the same time"));
 
   app.require_workspace();
 
   path_set paths;
-  if (app.unknown)
+  if (app.opts.unknown)
     {
-      path_restriction mask(args_to_paths(args), args_to_paths(app.exclude_patterns), app);
+      vector<file_path> roots = args_to_paths(args);
+      path_restriction mask(roots, args_to_paths(app.opts.exclude_patterns),
+                            app.opts.depth, app);
       path_set ignored;
-      find_unknown_and_ignored(app, mask, paths, ignored);
+
+      // if no starting paths have been specified use the workspace root
+      if (roots.empty())
+        roots.push_back(file_path());
+
+      app.work.find_unknown_and_ignored(mask, roots, paths, ignored);
     }
   else
-    for (vector<utf8>::const_iterator i = args.begin(); 
+    for (vector<utf8>::const_iterator i = args.begin();
          i != args.end(); ++i)
       {
         split_path sp;
@@ -243,31 +286,33 @@ CMD(add, N_("workspace"), N_("[PATH]..."),
         paths.insert(sp);
       }
 
-  bool add_recursive = !app.unknown;
-  perform_additions(paths, app, add_recursive);
+  bool add_recursive = app.opts.recursive;
+  app.work.perform_additions(paths, add_recursive, !app.opts.no_ignore);
 }
 
 CMD(drop, N_("workspace"), N_("[PATH]..."),
-    N_("drop files from workspace"), 
-    OPT_EXECUTE % OPT_MISSING % OPT_RECURSIVE)
+    N_("drop files from workspace"),
+    options::opts::execute | options::opts::missing | options::opts::recursive)
 {
-  if (!app.missing && (args.size() < 1))
+  if (!app.opts.missing && (args.size() < 1))
     throw usage(name);
 
   app.require_workspace();
 
   path_set paths;
-  if (app.missing)
+  if (app.opts.missing)
     {
       temp_node_id_source nis;
       roster_t current_roster_shape;
-      get_current_roster_shape(current_roster_shape, nis, app);
-      node_restriction mask(args_to_paths(args), args_to_paths(app.exclude_patterns),
+      app.work.get_current_roster_shape(current_roster_shape, nis);
+      node_restriction mask(args_to_paths(args),
+                            args_to_paths(app.opts.exclude_patterns),
+                            app.opts.depth,
                             current_roster_shape, app);
-      find_missing(current_roster_shape, mask, paths);
+      app.work.find_missing(current_roster_shape, mask, paths);
     }
   else
-    for (vector<utf8>::const_iterator i = args.begin(); 
+    for (vector<utf8>::const_iterator i = args.begin();
          i != args.end(); ++i)
       {
         split_path sp;
@@ -275,7 +320,7 @@ CMD(drop, N_("workspace"), N_("[PATH]..."),
         paths.insert(sp);
       }
 
-  perform_deletions(paths, app);
+  app.work.perform_deletions(paths, app.opts.recursive, app.opts.execute);
 }
 
 ALIAS(rm, drop);
@@ -285,7 +330,7 @@ CMD(rename, N_("workspace"),
     N_("SRC DEST\n"
        "SRC1 [SRC2 [...]] DEST_DIR"),
     N_("rename entries in the workspace"),
-    OPT_EXECUTE)
+    options::opts::execute)
 {
   if (args.size() < 2)
     throw usage(name);
@@ -300,7 +345,7 @@ CMD(rename, N_("workspace"),
       file_path s = file_path_external(idx(args, i));
       src_paths.insert(s);
     }
-  perform_rename(src_paths, dst_path, app);
+  app.work.perform_rename(src_paths, dst_path, app.opts.execute);
 }
 
 ALIAS(mv, rename)
@@ -314,7 +359,7 @@ ALIAS(mv, rename)
          "that is currently the root\n"
          "directory will have name PUT_OLD.\n"
          "Using --execute is strongly recommended."),
-      OPT_EXECUTE)
+    options::opts::execute)
 {
   if (args.size() != 2)
     throw usage(name);
@@ -322,14 +367,13 @@ ALIAS(mv, rename)
   app.require_workspace();
   file_path new_root = file_path_external(idx(args, 0));
   file_path put_old = file_path_external(idx(args, 1));
-  perform_pivot_root(new_root, put_old, app);
+  app.work.perform_pivot_root(new_root, put_old, app.opts.execute);
 }
 
-CMD(status, N_("informative"), N_("[PATH]..."), 
-    N_("show status of workspace"),
-    OPT_DEPTH % OPT_EXCLUDE % OPT_BRIEF)
+CMD(status, N_("informative"), N_("[PATH]..."), N_("show status of workspace"),
+    options::opts::depth | options::opts::exclude)
 {
-  roster_t old_roster, new_roster, restricted_roster;
+  roster_t old_roster, new_roster;
   cset included, excluded;
   revision_id old_rev_id;
   revision_t rev;
@@ -337,126 +381,108 @@ CMD(status, N_("informative"), N_("[PATH]..."),
   temp_node_id_source nis;
 
   app.require_workspace();
-  get_base_and_current_roster_shape(old_roster, new_roster, nis, app);
+  app.work.get_base_and_current_roster_shape(old_roster, new_roster, nis);
 
   node_restriction mask(args_to_paths(args),
-                        args_to_paths(app.exclude_patterns),
+                        args_to_paths(app.opts.exclude_patterns),
+                        app.opts.depth,
                         old_roster, new_roster, app);
 
-  update_current_roster_from_filesystem(new_roster, mask, app);
-  make_restricted_csets(old_roster, new_roster, 
+  app.work.update_current_roster_from_filesystem(new_roster, mask);
+  make_restricted_csets(old_roster, new_roster,
                         included, excluded, mask);
   check_restricted_cset(old_roster, included);
 
-  restricted_roster = old_roster;
-  editable_roster_base er(restricted_roster, nis);
-  included.apply_to(er);
+  app.work.get_revision_id(old_rev_id);
+  make_revision(old_rev_id, old_roster, included, rev);
 
-  get_revision_id(old_rev_id);
-  make_revision(old_rev_id, old_roster, restricted_roster, rev);
-
-  if (global_sanity.brief)
+  // We intentionally do not collapse the final \n into the format
+  // strings here, for consistency with newline conventions used by most
+  // other format strings.
+  cout << (F("Current branch: %s") % app.opts.branch_name).str() << "\n";
+  for (edge_map::const_iterator i = rev.edges.begin(); i != rev.edges.end(); ++i)
     {
-      I(rev.edges.size() == 1);
-      cset const & cs = edge_changes(rev.edges.begin());
+      revision_id parent = edge_old_revision(*i);
+      // A colon at the end of this string looked nicer, but it made
+      // double-click copying from terminals annoying.
+      cout << (F("Changes against parent %s") % parent).str() << "\n";
+
+      cset const & cs = edge_changes(*i);
+
+      if (cs.empty())
+        cout << F("  no changes").str() << "\n";
 
       for (path_set::const_iterator i = cs.nodes_deleted.begin();
-           i != cs.nodes_deleted.end(); ++i)
-        cout << "dropped " << *i << "\n";
+            i != cs.nodes_deleted.end(); ++i)
+        cout << (F("  dropped %s") % *i).str() << "\n";
 
       for (map<split_path, split_path>::const_iterator
-           i = cs.nodes_renamed.begin();
-           i != cs.nodes_renamed.end(); ++i)
-        cout << "renamed " << i->first << "\n"
-             << "     to " << i->second << "\n";
+            i = cs.nodes_renamed.begin();
+            i != cs.nodes_renamed.end(); ++i)
+        cout << (F("  renamed %s\n"
+                   "       to %s") % i->first % i->second).str() << "\n";
 
       for (path_set::const_iterator i = cs.dirs_added.begin();
-           i != cs.dirs_added.end(); ++i)
-        cout << "added   " << *i << "\n";
+            i != cs.dirs_added.end(); ++i)
+        cout << (F("  added   %s") % *i).str() << "\n";
 
       for (map<split_path, file_id>::const_iterator i = cs.files_added.begin();
-           i != cs.files_added.end(); ++i)
-        cout << "added   " << i->first << "\n";
+            i != cs.files_added.end(); ++i)
+        cout << (F("  added   %s") % i->first).str() << "\n";
 
       for (map<split_path, pair<file_id, file_id> >::const_iterator
-             i = cs.deltas_applied.begin(); i != cs.deltas_applied.end(); ++i)
-        cout << "patched " << i->first << "\n";
-    }
-  else
-    {
-      write_revision(rev, tmp);
-      cout << "\n" << tmp << "\n";
+              i = cs.deltas_applied.begin(); i != cs.deltas_applied.end(); ++i)
+        cout << (F("  patched %s") % (i->first)).str() << "\n";
     }
 }
 
-CMD(checkout, N_("tree"), N_("[DIRECTORY]\n"),
+CMD(checkout, N_("tree"), N_("[DIRECTORY]"),
     N_("check out a revision from database into directory.\n"
        "If a revision is given, that's the one that will be checked out.\n"
        "Otherwise, it will be the head of the branch (given or implicit).\n"
        "If no directory is given, the branch name will be used as directory"),
-    OPT_BRANCH_NAME % OPT_REVISION)
+    options::opts::branch | options::opts::revision)
 {
   revision_id ident;
   system_path dir;
-  // We have a special case for "checkout .", i.e., to current dir.
-  bool checkout_dot = false;
 
   transaction_guard guard(app.db, false);
 
-  if (args.size() > 1 || app.revision_selectors.size() > 1)
+  if (args.size() > 1 || app.opts.revision_selectors.size() > 1)
     throw usage(name);
 
-  if (args.size() == 0)
-    {
-      // No checkout dir specified, use branch name for dir.
-      N(!app.branch_name().empty(), 
-        F("need --branch argument for branch-based checkout"));
-      dir = system_path(app.branch_name());
-    }
-  else
-    {
-      // Checkout to specified dir.
-      dir = system_path(idx(args, 0));
-      if (idx(args, 0) == utf8("."))
-        checkout_dot = true;
-    }
-
-  if (!checkout_dot)
-    require_path_is_nonexistent
-      (dir, F("checkout directory '%s' already exists") % dir);
-
-  if (app.revision_selectors.size() == 0)
+  if (app.opts.revision_selectors.size() == 0)
     {
       // use branch head revision
-      N(!app.branch_name().empty(), 
-        F("need --branch argument for branch-based checkout"));
+      N(!app.opts.branch_name().empty(),
+        F("use --revision or --branch to specify what to checkout"));
 
       set<revision_id> heads;
-      get_branch_heads(app.branch_name(), app, heads);
-      N(heads.size() > 0, 
-        F("branch '%s' is empty") % app.branch_name);
+      get_branch_heads(app.opts.branch_name(), app, heads);
+      N(heads.size() > 0,
+        F("branch '%s' is empty") % app.opts.branch_name);
       if (heads.size() > 1)
         {
-          P(F("branch %s has multiple heads:") % app.branch_name);
+          P(F("branch %s has multiple heads:") % app.opts.branch_name);
           for (set<revision_id>::const_iterator i = heads.begin(); i != heads.end(); ++i)
             P(i18n_format("  %s") % describe_revision(app, *i));
-          P(F("choose one with '%s checkout -r<id>'") % app.prog_name);
-          E(false, F("branch %s has multiple heads") % app.branch_name);
+          P(F("choose one with '%s checkout -r<id>'") % ui.prog_name);
+          E(false, F("branch %s has multiple heads") % app.opts.branch_name);
         }
       ident = *(heads.begin());
     }
-  else if (app.revision_selectors.size() == 1)
+  else if (app.opts.revision_selectors.size() == 1)
     {
       // use specified revision
-      complete(app, idx(app.revision_selectors, 0)(), ident);
+      complete(app, idx(app.opts.revision_selectors, 0)(), ident);
       N(app.db.revision_exists(ident),
         F("no such revision '%s'") % ident);
 
       cert_value b;
       guess_branch(ident, app, b);
 
-      I(!app.branch_name().empty());
-      cert_value branch_name(app.branch_name());
+      I(!app.opts.branch_name().empty());
+      cert_value branch_name(app.opts.branch_name());
       base64<cert_value> branch_encoded;
       encode_base64(branch_name, branch_encoded);
 
@@ -465,26 +491,57 @@ CMD(checkout, N_("tree"), N_("[DIRECTORY]\n"),
 
       L(FL("found %d %s branch certs on revision %s")
         % certs.size()
-        % app.branch_name
+        % app.opts.branch_name
         % ident);
 
       N(certs.size() != 0, F("revision %s is not a member of branch %s")
-        % ident % app.branch_name);
+        % ident % app.opts.branch_name);
     }
 
+  // we do this part of the checking down here, because it is legitimate to
+  // do
+  //  $ mtn co -r h:net.venge.monotone
+  // and have mtn guess the branch, and then use that branch name as the
+  // default directory.  But in this case the branch name will not be set
+  // until after the guess_branch() call above:
+  {
+    bool checkout_dot = false;
+
+    if (args.size() == 0)
+      {
+        // No checkout dir specified, use branch name for dir.
+        N(!app.opts.branch_name().empty(),
+          F("you must specify a destination directory"));
+        dir = system_path(app.opts.branch_name());
+      }
+    else
+      {
+        // Checkout to specified dir.
+        dir = system_path(idx(args, 0));
+        if (idx(args, 0) == utf8("."))
+          checkout_dot = true;
+      }
+
+    if (!checkout_dot)
+      require_path_is_nonexistent
+        (dir, F("checkout directory '%s' already exists") % dir);
+  }
+
   app.create_workspace(dir);
 
   file_data data;
   roster_t ros;
   marking_map mm;
 
-  put_revision_id(ident);
-
   L(FL("checking out revision %s to directory %s") % ident % dir);
   app.db.get_roster(ident, ros, mm);
 
+  revision_t workrev;
+  make_revision_for_workspace(ident, cset(), workrev);
+  app.work.put_work_rev(workrev);
+
   node_map const & nodes = ros.all_nodes();
-  for (node_map::const_iterator i = nodes.begin(); 
+  for (node_map::const_iterator i = nodes.begin();
        i != nodes.end(); ++i)
     {
       node_t node = i->second;
@@ -511,9 +568,9 @@ CMD(checkout, N_("tree"), N_("[DIRECTORY]\n"),
           write_localized_data(path, dat.inner(), app.lua);
         }
     }
-  remove_work_cset();
-  update_any_attrs(app);
-  maybe_update_inodeprints(app);
+
+  app.work.update_any_attrs();
+  app.work.maybe_update_inodeprints();
   guard.commit();
 }
 
@@ -521,7 +578,7 @@ ALIAS(co, checkout)
 
 CMD(attr, N_("workspace"), N_("set PATH ATTR VALUE\nget PATH [ATTR]\ndrop PATH [ATTR]"),
     N_("set, get or drop file attributes"),
-    OPT_NONE)
+    options::opts::none)
 {
   if (args.size() < 2 || args.size() > 4)
     throw usage(name);
@@ -530,7 +587,7 @@ CMD(attr, N_("workspace"), N_("set PATH ATTR VALUE\nget PATH [ATTR]\ndrop PATH [
   temp_node_id_source nis;
 
   app.require_workspace();
-  get_base_and_current_roster_shape(old_roster, new_roster, nis, app);
+  app.work.get_base_and_current_roster_shape(old_roster, new_roster, nis);
 
 
   file_path path = file_path_external(idx(args,1));
@@ -566,18 +623,20 @@ CMD(attr, N_("workspace"), N_("set PATH ATTR VALUE\nget PATH [ATTR]\ndrop PATH [
             {
               attr_key a_key = idx(args, 2)();
               N(node->attrs.find(a_key) != node->attrs.end(),
-                F("Path '%s' does not have attribute '%s'\n")
+                F("Path '%s' does not have attribute '%s'")
                 % path % a_key);
               node->attrs[a_key] = make_pair(false, "");
             }
           else
             throw usage(name);
         }
+      revision_id base;
+      app.work.get_revision_id(base);
 
-      cset new_work;
-      make_cset(old_roster, new_roster, new_work);
-      put_work_cset(new_work);
-      update_any_attrs(app);
+      revision_t new_work;
+      make_revision_for_workspace(base, old_roster, new_roster, new_work);
+      app.work.put_work_rev(new_work);
+      app.work.update_any_attrs();
     }
   else if (subcmd == "get")
     {
@@ -588,8 +647,8 @@ CMD(attr, N_("workspace"), N_("set PATH ATTR VALUE\nget PATH [ATTR]\ndrop PATH [
                i != node->attrs.end(); ++i)
             if (i->second.first)
               {
-                cout << path << " : " 
-                     << i->first << "=" 
+                cout << path << " : "
+                     << i->first << "="
                      << i->second.second << "\n";
                 has_any_live_attrs = true;
               }
@@ -601,11 +660,11 @@ CMD(attr, N_("workspace"), N_("set PATH ATTR VALUE\nget PATH [ATTR]\ndrop PATH [
           attr_key a_key = idx(args, 2)();
           full_attr_map_t::const_iterator i = node->attrs.find(a_key);
           if (i != node->attrs.end() && i->second.first)
-            cout << path << " : " 
-                 << i->first << "=" 
+            cout << path << " : "
+                 << i->first << "="
                  << i->second.second << "\n";
           else
-            cout << (F("No attribute '%s' on path '%s'") 
+            cout << (F("No attribute '%s' on path '%s'")
                      % a_key % path) << "\n";
         }
       else
@@ -619,37 +678,33 @@ CMD(attr, N_("workspace"), N_("set PATH ATTR VALUE\nget PATH [ATTR]\ndrop PATH [
 
 CMD(commit, N_("workspace"), N_("[PATH]..."),
     N_("commit workspace to database"),
-    OPT_BRANCH_NAME % OPT_MESSAGE % OPT_MSGFILE % OPT_DATE %
-    OPT_AUTHOR % OPT_DEPTH % OPT_EXCLUDE)
+    options::opts::branch | options::opts::message | options::opts::msgfile | options::opts::date
+    | options::opts::author | options::opts::depth | options::opts::exclude)
 {
-  string log_message("");
+  utf8 log_message("");
   bool log_message_given;
   revision_t restricted_rev;
   revision_id old_rev_id, restricted_rev_id;
-  roster_t old_roster, new_roster, restricted_roster;
+  roster_t old_roster, new_roster;
   temp_node_id_source nis;
   cset included, excluded;
 
   app.make_branch_sticky();
   app.require_workspace();
-  get_base_and_current_roster_shape(old_roster, new_roster, nis, app);
+  app.work.get_base_and_current_roster_shape(old_roster, new_roster, nis);
 
   node_restriction mask(args_to_paths(args),
-                        args_to_paths(app.exclude_patterns),
+                        args_to_paths(app.opts.exclude_patterns),
+                        app.opts.depth,
                         old_roster, new_roster, app);
 
-  update_current_roster_from_filesystem(new_roster, mask, app);
-  make_restricted_csets(old_roster, new_roster, 
+  app.work.update_current_roster_from_filesystem(new_roster, mask);
+  make_restricted_csets(old_roster, new_roster,
                         included, excluded, mask);
   check_restricted_cset(old_roster, included);
 
-  restricted_roster = old_roster;
-  editable_roster_base er(restricted_roster, nis);
-  included.apply_to(er);
-
-  get_revision_id(old_rev_id);
-  make_revision(old_rev_id, old_roster, 
-                    restricted_roster, restricted_rev);
+  app.work.get_revision_id(old_rev_id);
+  make_revision(old_rev_id, old_roster, included, restricted_rev);
 
   calculate_ident(restricted_rev, restricted_rev_id);
 
@@ -659,14 +714,20 @@ CMD(commit, N_("workspace"), N_("[PATH]..."),
   I(restricted_rev.edges.size() == 1);
 
   set<revision_id> heads;
-  get_branch_heads(app.branch_name(), app, heads);
+  get_branch_heads(app.opts.branch_name(), app, heads);
   unsigned int old_head_size = heads.size();
 
-  if (app.branch_name() != "")
-    branchname = app.branch_name();
+  if (app.opts.branch_name() != "")
+    branchname = app.opts.branch_name();
   else
     guess_branch(edge_old_revision(restricted_rev.edges.begin()), app, branchname);
 
+  {
+    // fail early if there isn't a key
+    rsa_keypair_id key;
+    get_user_key(key, app);
+  }
+
   P(F("beginning commit on branch '%s'") % branchname);
   L(FL("new manifest '%s'\n"
        "new revision '%s'\n")
@@ -675,7 +736,7 @@ CMD(commit, N_("workspace"), N_("[PATH]..."),
 
   process_commit_message_args(log_message_given, log_message, app);
 
-  N(!(log_message_given && has_contents_user_log()),
+  N(!(log_message_given && app.work.has_contents_user_log()),
     F("_MTN/log is non-empty and log message "
       "was specified on command line\n"
       "perhaps move or delete _MTN/log,\n"
@@ -690,7 +751,7 @@ CMD(commit, N_("workspace"), N_("[PATH]..."),
       // We only check for empty log messages when the user entered them
       // interactively.  Consensus was that if someone wanted to explicitly
       // type --message="", then there wasn't any reason to stop them.
-      N(log_message.find_first_not_of("\n\r\t ") != string::npos,
+      N(log_message().find_first_not_of("\n\r\t ") != string::npos,
         F("empty log message; commit canceled"));
 
       // We save interactively entered log messages to _MTN/log, so if
@@ -701,18 +762,19 @@ CMD(commit, N_("workspace"), N_("[PATH]..."),
       // hit up-arrow to try again, you get an "_MTN/log non-empty and
       // message given on command line" error... which is annoying.
 
-      write_user_log(data(log_message));
+      app.work.write_user_log(log_message);
     }
 
   // If the hook doesn't exist, allow the message to be used.
   bool message_validated;
   string reason, new_manifest_text;
 
-  dump(restricted_rev, new_manifest_text);
+  revision_data new_rev;
+  write_revision(restricted_rev, new_rev);
 
-  app.lua.hook_validate_commit_message(log_message, new_manifest_text,
+  app.lua.hook_validate_commit_message(log_message, new_rev, branchname,
                                        message_validated, reason);
-  N(message_validated, F("log message rejected: %s") % reason);
+  N(message_validated, F("log message rejected by hook: %s") % reason);
 
   {
     transaction_guard guard(app.db);
@@ -734,7 +796,7 @@ CMD(commit, N_("workspace"), N_("[PATH]..."),
         // process file deltas or new files
         cset const & cs = edge_changes(edge);
 
-        for (map<split_path, pair<file_id, file_id> >::const_iterator 
+        for (map<split_path, pair<file_id, file_id> >::const_iterator
                i = cs.deltas_applied.begin();
              i != cs.deltas_applied.end(); ++i)
           {
@@ -770,12 +832,12 @@ CMD(commit, N_("workspace"), N_("[PATH]..."),
             else
               // If we don't err out here, our packet writer will
               // later.
-              E(false, 
+              E(false,
                 F("Your database is missing version %s of file '%s'")
                 % old_content % path);
           }
 
-        for (map<split_path, file_id>::const_iterator 
+        for (map<split_path, file_id>::const_iterator
                i = cs.files_added.begin();
              i != cs.files_added.end(); ++i)
           {
@@ -800,13 +862,13 @@ CMD(commit, N_("workspace"), N_("[PATH]..."),
     dbw.consume_revision_data(restricted_rev_id, rdat);
 
     cert_revision_in_branch(restricted_rev_id, branchname, app, dbw);
-    if (app.date_set)
-      cert_revision_date_time(restricted_rev_id, app.date, app, dbw);
+    if (app.opts.date_given)
+      cert_revision_date_time(restricted_rev_id, app.opts.date, app, dbw);
     else
       cert_revision_date_now(restricted_rev_id, app, dbw);
 
-    if (app.author().length() > 0)
-      cert_revision_author(restricted_rev_id, app.author(), app, dbw);
+    if (app.opts.author_given > 0)
+      cert_revision_author(restricted_rev_id, app.opts.author(), app, dbw);
     else
       cert_revision_author_default(restricted_rev_id, app, dbw);
 
@@ -814,22 +876,26 @@ CMD(commit, N_("workspace"), N_("[PATH]..."),
     guard.commit();
   }
 
+  // the work revision is now whatever changes remain on top of the revision
+  // we just checked in.
+  revision_t remaining;
+  make_revision_for_workspace(restricted_rev_id, excluded, remaining);
+
   // small race condition here...
-  put_work_cset(excluded);
-  put_revision_id(restricted_rev_id);
+  app.work.put_work_rev(remaining);
   P(F("committed revision %s") % restricted_rev_id);
 
-  blank_user_log();
+  app.work.blank_user_log();
 
-  get_branch_heads(app.branch_name(), app, heads);
+  get_branch_heads(app.opts.branch_name(), app, heads);
   if (heads.size() > old_head_size && old_head_size > 0) {
     P(F("note: this revision creates divergence\n"
         "note: you may (or may not) wish to run '%s merge'")
-      % app.prog_name);
+      % ui.prog_name);
   }
 
-  update_any_attrs(app);
-  maybe_update_inodeprints(app);
+  app.work.update_any_attrs();
+  app.work.maybe_update_inodeprints();
 
   {
     // Tell lua what happened. Yes, we might lose some information
@@ -856,14 +922,14 @@ CMD(commit, N_("workspace"), N_("[PATH]..."),
 ALIAS(ci, commit);
 
 
-CMD_NO_WORKSPACE(setup, N_("tree"), N_("[DIRECTORY]"), 
-                 N_("setup a new workspace directory, default to current"),
-                 OPT_BRANCH_NAME)
+CMD_NO_WORKSPACE(setup, N_("tree"), N_("[DIRECTORY]"),
+    N_("setup a new workspace directory, default to current"),
+    options::opts::branch)
 {
   if (args.size() > 1)
     throw usage(name);
 
-  N(!app.branch_name().empty(), F("need --branch argument for setup"));
+  N(!app.opts.branch_name().empty(), F("need --branch argument for setup"));
   app.db.ensure_open();
 
   string dir;
@@ -873,17 +939,32 @@ CMD_NO_WORKSPACE(setup, N_("tree"), N_("[DIRECTORY]"),
     dir = ".";
 
   app.create_workspace(dir);
-  revision_id null;
-  put_revision_id(null);
+
+  revision_t rev;
+  make_revision_for_workspace(revision_id(), cset(), rev);
+  app.work.put_work_rev(rev);
 }
 
-CMD(refresh_inodeprints, N_("tree"), "", 
-    N_("refresh the inodeprint cache"),
-    OPT_NONE)
+CMD_NO_WORKSPACE(migrate_workspace, N_("tree"), N_("[DIRECTORY]"),
+  N_("migrate a workspace directory's metadata to the latest format; "
+     "defaults to the current workspace"),
+  options::opts::none)
+{
+  if (args.size() > 1)
+    throw usage(name);
+
+  if (args.size() == 1)
+    go_to_workspace(system_path(idx(args, 0)));
+
+  app.work.migrate_ws_format();
+}
+
+CMD(refresh_inodeprints, N_("tree"), "", N_("refresh the inodeprint cache"),
+    options::opts::none)
 {
   app.require_workspace();
-  enable_inodeprints();
-  maybe_update_inodeprints(app);
+  app.work.enable_inodeprints();
+  app.work.maybe_update_inodeprints();
 }
 
 
