@@ -15,9 +15,14 @@
 #include "charset.hh"
 #include "inodeprint.hh"
 #include "cert.hh"
-
+#include "ui.hh"
 #include "cmd.hh"
 
+#ifndef _WIN32
+#include <boost/lexical_cast.hpp>
+#include <signal.h>
+#endif
+
 using std::cin;
 using std::pair;
 using std::set;
@@ -25,7 +30,6 @@ using std::string;
 using std::strlen;
 using std::vector;
 
-//
 // this file defines the task-oriented "top level" commands which can be
 // issued as part of a monotone command line. the command line can only
 // have one such command on it, followed by a vector of strings which are its
@@ -52,27 +56,33 @@ namespace commands
   // guarantee about what order they'll be initialized in. So have this
   // be something that doesn't get automatic initialization, and initialize
   // it ourselves the first time we use it.
-  static map<string,command *> *cmds;
+  static map<string, command *> * cmds;
   command::command(string const & n,
-                  string const & g,
-                  string const & p,
-                  string const & d,
-                  bool u,
-                  command_opts const & o)
+                   string const & g,
+                   string const & p,
+                   string const & d,
+                   bool u,
+                   options::options_type const & o)
     : name(n), cmdgroup(g), params_(p), desc_(d), use_workspace_options(u),
       options(o)
   {
-    static bool first(true);
-    if (first)
+    if (cmds == NULL)
       cmds = new map<string, command *>;
-    first = false;
     (*cmds)[n] = this;
   }
   command::~command() {}
   std::string command::params() {return safe_gettext(params_.c_str());}
   std::string command::desc() {return safe_gettext(desc_.c_str());}
+  options::options_type command::get_options(vector<utf8> const & args)
+  {
+    return options;
+  }
   bool operator<(command const & self, command const & other);
-  const std::string hidden_group("");
+  std::string const & hidden_group()
+  {
+    static const std::string the_hidden_group("");
+    return the_hidden_group;
+  }
 };
 
 namespace std
@@ -133,7 +143,7 @@ namespace commands
       }
 
     // more than one matched command
-    string err = (F("command '%s' has multiple ambiguous expansions:\n") % cmd).str();
+    string err = (F("command '%s' has multiple ambiguous expansions:") % cmd).str();
     for (vector<string>::iterator i = matched.begin();
          i != matched.end(); ++i)
       err += (*i + "\n");
@@ -169,7 +179,7 @@ namespace commands
     out << _("commands:") << endl;
     for (i = (*cmds).begin(); i != (*cmds).end(); ++i)
       {
-        if (i->second->cmdgroup != hidden_group)
+        if (i->second->cmdgroup != hidden_group())
           sorted.push_back(i->second);
       }
 
@@ -229,27 +239,41 @@ namespace commands
       }
   }
 
-  set<int> command_options(string const & cmd)
+  options::options_type command_options(vector<utf8> const & cmdline)
   {
+    if (cmdline.empty())
+      return options::options_type();
+    string cmd = complete_command(idx(cmdline,0)());
     if ((*cmds).find(cmd) != (*cmds).end())
       {
-        return (*cmds)[cmd]->options.opts;
+        return (*cmds)[cmd]->get_options(cmdline);
       }
     else
       {
-        return set<int>();
+        return options::options_type();
       }
   }
 
-  const no_opts OPT_NONE = no_opts();
+  options::options_type toplevel_command_options(string const & cmd)
+  {
+    if ((*cmds).find(cmd) != (*cmds).end())
+      {
+        return (*cmds)[cmd]->options;
+      }
+    else
+      {
+        return options::options_type();
+      }
+  }
 }
 ////////////////////////////////////////////////////////////////////////
 
-CMD(help, N_("informative"), N_("command [ARGS...]"), N_("display command help"), OPT_NONE)
+CMD(help, N_("informative"), N_("command [ARGS...]"),
+    N_("display command help"), options::opts::none)
 {
   if (args.size() < 1)
     {
-      app.requested_help = true;
+      app.opts.help = true;
       throw usage("");
     }
 
@@ -257,11 +281,12 @@ CMD(help, N_("informative"), N_("command [ARGS...]"), N_("display command help")
   if ((*cmds).find(full_cmd) == (*cmds).end())
     throw usage("");
 
-  app.requested_help = true;
+  app.opts.help = true;
   throw usage(full_cmd);
 }
 
-CMD(crash, hidden_group, "{ N | E | I }", "trigger the specified kind of crash", OPT_NONE)
+CMD(crash, hidden_group(), "{ N | E | I | exception | signal }",
+    "trigger the specified kind of crash", options::opts::none)
 {
   if (args.size() != 1)
     throw usage(name);
@@ -274,52 +299,43 @@ CMD(crash, hidden_group, "{ N | E | I }", "trigger the specified kind of crash",
     {
       I(spoon_exists);
     }
+#define maybe_throw(ex) if(idx(args,0)()==#ex) throw ex("There is no spoon.")
+#define maybe_throw_bare(ex) if(idx(args,0)()==#ex) throw ex()
+  else maybe_throw_bare(std::bad_alloc);
+  else maybe_throw_bare(std::bad_cast);
+  else maybe_throw_bare(std::bad_typeid);
+  else maybe_throw_bare(std::bad_exception);
+  else maybe_throw_bare(std::exception);
+  else maybe_throw(std::domain_error);
+  else maybe_throw(std::invalid_argument);
+  else maybe_throw(std::length_error);
+  else maybe_throw(std::out_of_range);
+  else maybe_throw(std::range_error);
+  else maybe_throw(std::overflow_error);
+  else maybe_throw(std::underflow_error);
+  else maybe_throw(std::logic_error);
+  else maybe_throw(std::runtime_error);
   else
-    throw usage(name);
-}
-
-void
-maybe_update_inodeprints(app_state & app)
-{
-  if (!in_inodeprints_mode())
-    return;
-  inodeprint_map ipm_new;
-  temp_node_id_source nis;
-  roster_t old_roster, new_roster;
-
-  get_base_and_current_roster_shape(old_roster, new_roster, nis, app);
-  update_current_roster_from_filesystem(new_roster, app);
-
-  node_map const & new_nodes = new_roster.all_nodes();
-  for (node_map::const_iterator i = new_nodes.begin(); i != new_nodes.end(); ++i)
     {
-      node_id nid = i->first;
-      if (old_roster.has_node(nid))
+#ifndef _WIN32
+      try
         {
-          node_t old_node = old_roster.get_node(nid);
-          if (is_file_t(old_node))
+          int signo = boost::lexical_cast<int>(idx(args,0)());
+          if (0 < signo && signo <= 15)
             {
-              node_t new_node = i->second;
-              I(is_file_t(new_node));
-
-              file_t old_file = downcast_to_file_t(old_node);
-              file_t new_file = downcast_to_file_t(new_node);
-
-              if (new_file->content == old_file->content)
-                {
-                  split_path sp;
-                  new_roster.get_name(nid, sp);
-                  file_path fp(sp);
-                  hexenc<inodeprint> ip;
-                  if (inodeprint_file(fp, ip))
-                    ipm_new.insert(inodeprint_entry(fp, ip));
-                }
+              raise(signo);
+              // control should not get here...
+              I(!"crash: raise returned");
             }
         }
+      catch (boost::bad_lexical_cast&)
+        { // fall through and throw usage
+        }
+#endif
+      throw usage(name);
     }
-  data dat;
-  write_inodeprint_map(ipm_new, dat);
-  write_inodeprints(dat);
+#undef maybe_throw
+#undef maybe_throw_bare
 }
 
 string
@@ -428,10 +444,10 @@ complete(app_state & app,
 
   if (completions.size() > 1)
     {
-      string err = (F("selection '%s' has multiple ambiguous expansions: \n") % str).str();
+      string err = (F("selection '%s' has multiple ambiguous expansions:") % str).str();
       for (set<revision_id>::const_iterator i = completions.begin();
            i != completions.end(); ++i)
-        err += (describe_revision(app, *i) + "\n");
+        err += ("\n" + describe_revision(app, *i));
       N(completions.size() == 1, i18n_format(err));
     }
 
@@ -442,40 +458,49 @@ void
 notify_if_multiple_heads(app_state & app)
 {
   set<revision_id> heads;
-  get_branch_heads(app.branch_name(), app, heads);
+  get_branch_heads(app.opts.branch_name(), app, heads);
   if (heads.size() > 1) {
     string prefixedline;
     prefix_lines_with(_("note: "),
                       _("branch '%s' has multiple heads\n"
                         "perhaps consider '%s merge'"),
                       prefixedline);
-    P(i18n_format(prefixedline) % app.branch_name % app.prog_name);
+    P(i18n_format(prefixedline) % app.opts.branch_name % ui.prog_name);
   }
 }
 
-// FIXME BUG: our log message handling is terribly locale-unaware -- if it's
-// passed as -m, we convert to unicode, if it's passed as --message-file or
-// entered interactively, we simply pass it through as bytes.
-
 void
 process_commit_message_args(bool & given,
-                            string & log_message,
-                            app_state & app)
+                            utf8 & log_message,
+                            app_state & app,
+                            utf8 message_prefix)
 {
   // can't have both a --message and a --message-file ...
-  N(app.message().length() == 0 || app.message_file().length() == 0,
+  N(!app.opts.message_given || !app.opts.msgfile_given,
     F("--message and --message-file are mutually exclusive"));
 
-  if (app.is_explicit_option(OPT_MESSAGE))
+  if (app.opts.message_given)
     {
-      log_message = app.message();
+      std::string msg;
+      join_lines(app.opts.message, msg);
+      log_message = utf8(msg);
+      if (message_prefix().length() != 0)
+        log_message = message_prefix() + "\n\n" + log_message();
       given = true;
     }
-  else if (app.is_explicit_option(OPT_MSGFILE))
+  else if (app.opts.msgfile_given)
     {
       data dat;
-      read_data_for_command_line(app.message_file(), dat);
-      log_message = dat();
+      read_data_for_command_line(app.opts.msgfile, dat);
+      external dat2 = dat();
+      system_to_utf8(dat2, log_message);
+      if (message_prefix().length() != 0)
+        log_message = message_prefix() + "\n\n" + log_message();
+      given = true;
+    }
+  else if (message_prefix().length() != 0)
+    {
+      log_message = message_prefix;
       given = true;
     }
   else
