@@ -1,3 +1,4 @@
+// -*- mode: C++; c-file-style: "gnu"; indent-tabs-mode: nil -*-
 // Timothy Brownawell  <tbrownaw@gmail.com>
 // GPL v2
 //
@@ -15,17 +16,19 @@
 // Usage: usher [-l address[:port]] [-a address:port] [-p pidfile] <server-file>
 //
 // options:
-// -l   address and port to listen on, defaults to 0.0.0.0:5253
+// -m   the monotone command, defaults to "mtn"
+// -l   address and port to listen on, defaults to 0.0.0.0:4691
 // -a   address and port to listen for admin commands
 // -p   a file (deleted on program exit) to record the pid of the usher in
 // <server-file>   a file that looks like
 //   userpass username password
+//   logdir directory
 //
 //   server monotone
 //   host localhost
 //   pattern net.venge.monotone
-//   remote 66.96.28.3:5253
-//   
+//   remote 66.96.28.3:4691
+//
 //   server local
 //   host 127.0.0.1
 //   pattern *
@@ -33,6 +36,7 @@
 //
 // or in general, one block of one or more lines of
 //   userpass <username> <password>
+//   logdir <directory>
 // followed by any number of blocks of a
 //   server <name>
 // line followed by one or more
@@ -49,8 +53,8 @@
 // A request to server "hostname" will be directed to the
 // server at <ip-address>:<port-number>, if that stem is marked as remote,
 // and to a local server managed by the usher, started with the given
-// arguments ("monotone serve --bind=something <server arguments>"),
-// if it is marked as local.
+// arguments ("mtn serve --bind=something <server arguments>"), if it is
+// marked as local.
 // Note that "hostname" has to be an initial substring of who the client asked
 // to connect to, but does not have to match exactly. This means that you don't
 // have to know in advance whether clients will be asking for
@@ -123,6 +127,8 @@
 #include <unistd.h>
 #include <sys/time.h>
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <sys/wait.h>
 #include <string.h>
 #include <signal.h>
@@ -131,6 +137,7 @@
 #include <arpa/inet.h>
 #include <errno.h>
 #include <time.h>
+#include <locale.h>
 
 #include <string>
 #include <list>
@@ -157,8 +164,10 @@ using std::pair;
 using std::make_pair;
 
 // defaults, overridden by command line
-int listenport = 5253;
+int listenport = 4691;
 string listenaddr = "0.0.0.0";
+string monotone = "mtn";
+string logdir = ".";
 
 // keep local servers around for this many seconds after the last
 // client disconnects from them (only accurate to ~10 seconds)
@@ -172,7 +181,7 @@ int const maxaddr[] = {127, 254, 254, 254};
 int currport = 0;
 int curraddr[] = {0, 0, 0, 0};
 
-char const netsync_version = 5;
+char const netsync_version = 6;
 
 string const greeting = " Hello! This is the monotone usher at localhost. What would you like?";
 
@@ -486,11 +495,17 @@ sock make_outgoing(int port, string const & address)
   return s;
 }
 
-int fork_server(vector<string> const & args)
+int fork_server(const string & name, vector<string> const & args)
 {
+  string logfile = logdir + "/" + name + ".log";
   int err[2];
-  if (pipe(err) < 0)
+  if ((err[1]=open(logfile.c_str(),O_CREAT|O_APPEND,0644)) < 0)
     return false;
+  if ((err[0]=open(logfile.c_str(),O_RDONLY)) < 0)
+    {
+      close(err[1]);
+      return false;
+    }
   int pid = fork();
   if (pid == -1) {
     close(err[0]);
@@ -503,7 +518,11 @@ int fork_server(vector<string> const & args)
     close(1);
     close(2);
     sock::close_all_socks();
-    if (dup2(err[1], 2) < 0) {
+    if (dup2(err[1], 1) < 0) {
+      exit(1);
+    }
+    close(err[1]);
+    if (dup2(1, 2) < 0) {
       exit(1);
     }
 
@@ -514,8 +533,15 @@ int fork_server(vector<string> const & args)
     }
     a[args.size()] = 0;
 
+    // Must set C locale, because usher interprets the output from
+    // monotone itself, and only deals with english!
+    setlocale(LC_ALL,"C");
+    setlocale(LC_MESSAGES,"C");
+    setlocale(LC_NUMERIC,"C");
+    setenv("LC_ALL","C",1);
+
     execvp(a[0], a);
-    perror("execvp failed\n");
+    perror("execvp failed");
     exit(1);
   } else {
     close(err[1]);
@@ -526,7 +552,7 @@ int fork_server(vector<string> const & args)
     // the first line output on the server's stderr will be either
     // "monotone: beginning service on <interface> : <port>" or
     // "monotone: network error: bind(2) error: Address already in use"
-    while(r >= 0 && !line && got < 256) {
+    do {
       r = read(err[0], head + got, 256 - got);
       if (r)
         cerr<<"Read '"<<string(head+got, r)<<"'\n";
@@ -536,7 +562,8 @@ int fork_server(vector<string> const & args)
             line = true;
         got += r;
       }
-    }
+    } while(r >= 0 && !line && got < 256);
+    close(err[0]);
     head[got] = 0;
     if (string(head).find("beginning service") != string::npos)
       return pid;
@@ -688,12 +715,25 @@ struct server
     for (vector<string>::const_iterator i = h.begin(); i != h.end(); ++i) {
       c = servers_by_host.find(*i);
       if (c != servers_by_host.end()) {
+        cerr << "Removing duplicate for hostname " << *i << " in:" << std::endl
+             << "  ";
         list<map<string, shared_ptr<server> >::iterator>::iterator j;
-        for (j = c->second->by_host.begin(); j != c->second->by_host.end(); ++j)
-          if ((*j)->first == *i) {
-            servers_by_host.erase(*j);
-            c->second->by_host.erase(j);
+        bool first = true;
+        for (j = c->second->by_host.begin(); j != c->second->by_host.end();)
+          {
+            list<map<string, shared_ptr<server> >::iterator>::iterator j_saved
+              = j;
+            ++j;
+            cerr << (first ? "" : ", ") << (*j_saved)->second->by_name->first;
+            first = false;
+            if ((*j_saved)->first == *i)
+              {
+                servers_by_host.erase(*j_saved);
+                c->second->by_host.erase(j_saved);
+              }
           }
+        cerr << std::endl
+             << "  ... because it appeared in " << by_name->first << std::endl;
       }
       c = servers_by_host.insert(make_pair(*i, me)).first;
       by_host.push_back(c);
@@ -710,12 +750,25 @@ struct server
     for (vector<string>::const_iterator i = p.begin(); i != p.end(); ++i) {
       c = servers_by_pattern.find(*i);
       if (c != servers_by_pattern.end()) {
+        cerr << "Removing duplicate for pattern " << *i << " in: " << std::endl
+             << "  ";
         list<map<string, shared_ptr<server> >::iterator>::iterator j;
+        bool first = true;
         for (j = c->second->by_pat.begin(); j != c->second->by_pat.end(); ++j)
-          if ((*j)->first == *i) {
-            servers_by_pattern.erase(*j);
-            c->second->by_pat.erase(j);
+          {
+            list<map<string, shared_ptr<server> >::iterator>::iterator j_saved
+              = j;
+            ++j;
+            cerr << (first ? "" : ", ") << (*j_saved)->second->by_name->first;
+            first = false;
+            if ((*j_saved)->first == *i)
+              {
+                servers_by_pattern.erase(*j_saved);
+                c->second->by_pat.erase(j_saved);
+              }
           }
+        cerr << std::endl
+             << "  ... because it appeared in " << by_name->first << std::endl;
       }
       c = servers_by_pattern.insert(make_pair(*i, me)).first;
       by_pat.push_back(c);
@@ -735,8 +788,9 @@ struct server
         if (i > 0 || port == 0)
           find_addr(addr, port);
         vector<string> args;
-        args.push_back("monotone");
+        args.push_back(monotone);
         args.push_back("serve");
+        args.push_back("--ticker=dot");
         args.push_back("--bind=" + addr + ":" + lexical_cast<string>(port));
         unsigned int n = 0, m = 0;
         n = arguments.find_first_not_of(" \t");
@@ -745,7 +799,7 @@ struct server
           args.push_back(arguments.substr(n, m-n));
           n = arguments.find_first_not_of(" ", m);
         }
-        pid = fork_server(args);
+        pid = fork_server(by_name->first,args);
       }
     }
     sock s = make_outgoing(port, addr);
@@ -853,6 +907,10 @@ string read_server_record(std::istream & in)
       hosts.push_back(arg);
     else if (cmd == "pattern")
       patterns.push_back(arg);
+    else
+      cerr << "Unrecognised directive " << cmd << ", skipping line..."
+           << std::endl;
+
     line = getline(in);
   }
   if (name.empty())
@@ -882,7 +940,7 @@ string read_server_record(std::istream & in)
       srv->port = lexical_cast<int>(desc.substr(c+1));
     } else {
       srv->addr = desc;
-      srv->port = 5253;
+      srv->port = 4691;
     }
   }
   return name;
@@ -1139,9 +1197,20 @@ void reload_conffile(string const & file)
   while (!line.empty()) {
     std::istringstream iss(line);
     string a, b, c;
-    iss>>a>>b>>c;
+    iss>>a;
     if (a == "userpass")
-      admins.insert(make_pair(b, c));
+      {
+        iss>>b>>c;
+        admins.insert(make_pair(b, c));
+      }
+    else if (a == "logdir")
+      {
+        iss>>b;
+        logdir = b;
+      }
+    else
+      cerr << "Unrecognised directive " << a << ", skipping line..."
+           << std::endl;
     line = getline(cf);
   }
 
@@ -1209,7 +1278,7 @@ struct administrator
         if (cs.auth == true)
           return false;
         cs.auth = true;
-        process(cs);
+        return process(cs);
       }
     } else if (cmd == "STATUS") {
       string srv;
@@ -1286,6 +1355,8 @@ struct administrator
     } else if (cmd == "STARTUP") {
       connections_allowed = true;
       cs.buf = "ok\n";
+    } else {
+      return true;
     }
     cs.rdone = true;
     return true;
@@ -1311,10 +1382,10 @@ struct administrator
     for (list<pair<cstate, sock> >::iterator i = conns.begin();
          i != conns.end(); ++i) {
       int c = i->second;
-      if (i->first.rdone)
-        FD_SET(c, &wr);
-      else
+      if (!i->first.rdone)
         FD_SET(c, &rd);
+      else
+        FD_SET(c, &wr);
       maxfd = max(maxfd, int(c));
     }
   }
@@ -1334,35 +1405,46 @@ struct administrator
         cerr<<"During new admin connection: "<<s.name<<"\n";
       }
     }
+    list<list<pair<cstate, sock> >::iterator> del;
     for (list<pair<cstate, sock> >::iterator i = conns.begin();
          i != conns.end(); ++i) {
       int c = i->second;
-      if (c <= 0)
-        conns.erase(i);
-      else if (FD_ISSET(c, &rd)) {
+      if (c <= 0) {
+//        cerr<<"Bad socket.\n";
+        del.push_back(i);
+      } else if (FD_ISSET(c, &rd)) {
         char buf[120];
         int n;
         n = read(c, buf, 120);
-        if (n < 1)
-          conns.erase(i);
+        if (n < 1) {
+ //         cerr<<"Read failed.\n";
+          del.push_back(i);
+        }
         i->first.buf.append(buf, n);
         if (!process(i->first)) {
-          cerr<<"Closing connection...\n";
+//          cerr<<"Closing connection...\n";
 //          i->second.close();
-          conns.erase(i);
+          del.push_back(i);
         }
       }
       else if (FD_ISSET(c, &wr)) {
         int n = write(c, i->first.buf.c_str(), i->first.buf.size());
-        if (n < 1)
-          conns.erase(i);
-        else {
+        if (n < 1) {
+//          cerr<<"Write failed.\n";
+          del.push_back(i);
+        } else {
           i->first.buf.erase(0, n);
-          if (i->first.buf.empty())
-            conns.erase(i);
+          if (i->first.buf.empty() && i->first.rdone) {
+//            cerr<<"Done.\n";
+            del.push_back(i);
+          }
         }
       }
     }
+    for (list<list<pair<cstate, sock> >::iterator>::iterator i = del.begin();
+         i != del.end(); ++i) {
+      conns.erase(*i);
+    }
   }
 };
 
@@ -1401,7 +1483,9 @@ int main (int argc, char **argv)
         listenaddr = lp.substr(0, c);
         if (c != lp.npos)
           listenport = lexical_cast<int>(lp.substr(c+1));
-      } else if (string(argv[i]) == "-a")
+      } else if (string(argv[i]) == "-m")
+        monotone = argv[++i];
+      else if (string(argv[i]) == "-a")
         admin.initialize(argv[++i]);
       else if (string(argv[i]) == "-p")
         pf.initialize(argv[++i]);
