@@ -92,7 +92,7 @@ using boost::shared_ptr;
 //
 
 typedef enum { preserved = 0, deleted = 1, changed = 2 } edit_t;
-static char *etab[3] =
+static const char etab[3][10] =
   {
     "preserved",
     "deleted",
@@ -520,24 +520,25 @@ content_merge_database_adaptor::record_merge(file_id const & left_ident,
 
 static void
 load_and_cache_roster(revision_id const & rid,
-                      map<revision_id, shared_ptr<roster_t> > & rmap,
-                      shared_ptr<roster_t> & rout,
+                      map<revision_id, shared_ptr<roster_t const> > & rmap,
+                      shared_ptr<roster_t const> & rout,
                       app_state & app)
 {
-  map<revision_id, shared_ptr<roster_t> >::const_iterator i = rmap.find(rid);
+  map<revision_id, shared_ptr<roster_t const> >::const_iterator i = rmap.find(rid);
   if (i != rmap.end())
     rout = i->second;
   else
     {
-      rout = shared_ptr<roster_t>(new roster_t());
-      app.db.get_roster(rid, *rout);
-      safe_insert(rmap, make_pair(rid, rout));
+      database::cached_roster cr;
+      app.db.get_roster(rid, cr);
+      safe_insert(rmap, make_pair(rid, cr.first));
+      rout = cr.first;
     }
 }
 
 void
 content_merge_database_adaptor::get_ancestral_roster(node_id nid,
-                                                     shared_ptr<roster_t> & anc)
+                                                     shared_ptr<roster_t const> & anc)
 {
   // Given a file, if the lca is nonzero and its roster contains the file,
   // then we use its roster.  Otherwise we use the roster at the file's
@@ -561,7 +562,7 @@ content_merge_database_adaptor::get_ancestral_roster(node_id nid,
 void
 content_merge_database_adaptor::get_version(file_path const & path,
                                             file_id const & ident,
-                                            file_data & dat)
+                                            file_data & dat) const
 {
   app.db.get_file_version(ident, dat);
 }
@@ -580,13 +581,15 @@ content_merge_workspace_adaptor::record_merge(file_id const & left_id,
 {
   L(FL("temporarily recording merge of %s <-> %s into %s")
     % left_id % right_id % merged_id);
-  I(temporary_store.find(merged_id) == temporary_store.end());
+  // this is an insert instead of a safe_insert because it is perfectly
+  // legal (though rare) to have multiple merges resolve to the same file
+  // contents.
   temporary_store.insert(make_pair(merged_id, merged_data));
 }
 
 void
 content_merge_workspace_adaptor::get_ancestral_roster(node_id nid,
-                                                      shared_ptr<roster_t> & anc)
+                                                      shared_ptr<roster_t const> & anc)
 {
   // When doing an update, the base revision is always the ancestor to
   // use for content merging.
@@ -596,9 +599,12 @@ content_merge_workspace_adaptor::get_ancestral_roster(node_id nid,
 void
 content_merge_workspace_adaptor::get_version(file_path const & path,
                                              file_id const & ident,
-                                             file_data & dat)
+                                             file_data & dat) const
 {
-  if (app.db.file_version_exists(ident))
+  map<file_id,file_data>::const_iterator i = temporary_store.find(ident);
+  if (i != temporary_store.end())
+    dat = i->second;
+  else if (app.db.file_version_exists(ident))
     app.db.get_file_version(ident, dat);
   else
     {
@@ -609,7 +615,7 @@ content_merge_workspace_adaptor::get_version(file_path const & path,
                            F("'%s' in workspace is a directory, not a file") % path);
       read_localized_data(path, tmp, app.lua);
       calculate_ident(tmp, fid);
-      N(fid == ident,
+      E(fid == ident,
         F("file %s in workspace has id %s, wanted %s")
         % path % fid % ident);
       dat = tmp;
@@ -638,7 +644,9 @@ content_merger::get_file_encoding(file_path const & path,
                                   roster_t const & ros)
 {
   attr_value v;
-  if (get_attribute_from_roster(ros, path, constants::encoding_attribute, v))
+  split_path sp;
+  path.split(sp);
+  if (ros.get_attr(sp, constants::encoding_attribute, v))
     return v();
   return constants::default_encoding;
 }
@@ -648,7 +656,9 @@ content_merger::attribute_manual_merge(file_path const & path,
                                        roster_t const & ros)
 {
   attr_value v;
-  if (get_attribute_from_roster(ros, path, constants::manual_merge_attribute, v)
+  split_path sp;
+  path.split(sp);
+  if (ros.get_attr(sp, constants::manual_merge_attribute, v)
       && v() == "true")
     return true;
   return false; // default: enable auto merge
@@ -735,7 +745,7 @@ content_merger::try_to_merge_files(file_path const & anc_path,
       "[ancestor] %s\n"
       "[    left] %s\n"
       "[   right] %s\n"
-      "[  merged] %s\n")
+      "[  merged] %s")
     % anc_path
     % left_path
     % right_path
@@ -776,8 +786,8 @@ struct hunk_consumer
   size_t a_begin, b_begin, a_len, b_len;
   long skew;
 
-  vector<string>::const_reverse_iterator encloser_last_search;
   vector<string>::const_reverse_iterator encloser_last_match;
+  vector<string>::const_reverse_iterator encloser_last_search;
 
   virtual void flush_hunk(size_t pos) = 0;
   virtual void advance_to(size_t newpos) = 0;
@@ -802,7 +812,7 @@ struct hunk_consumer
 /* Find, and write to ENCLOSER, the nearest line before POS which matches
    ENCLOSER_PATTERN.  We remember the last line scanned, and the matched, to
    avoid duplication of effort.  */
-   
+
 void
 hunk_consumer::find_encloser(size_t pos, string & encloser)
 {
@@ -810,7 +820,10 @@ hunk_consumer::find_encloser(size_t pos, string & encloser)
 
   // Precondition: encloser_last_search <= pos <= a.size().
   I(pos <= a.size());
-  I(pos >= a.rend() - encloser_last_search);
+  // static_cast<> to silence compiler unsigned vs. signed comparison
+  // warning, after first making sure that the static_cast is safe.
+  I(a.rend() - encloser_last_search >= 0);
+  I(pos >= static_cast<size_t>(a.rend() - encloser_last_search));
 
   if (!encloser_re)
     return;
@@ -960,7 +973,7 @@ void unidiff_hunk_writer::flush_hunk(size_t pos)
               first_mod = i - hunk.begin();
               break;
             }
-        
+
         find_encloser(a_begin + first_mod, encloser);
         ost << " @@" << encloser << endl;
       }
@@ -1097,7 +1110,7 @@ void cxtdiff_hunk_writer::flush_hunk(size_t pos)
 
         find_encloser(a_begin + min(first_insert, first_delete),
                       encloser);
-        
+
         ost << "***************" << encloser << endl;
       }
 
@@ -1269,7 +1282,7 @@ make_diff(string const & filename1,
   //   If there is a tab, considers everything up to that tab to be the
   //   filename.  If there is not a tab, considers everything up to the
   //   first whitespace to be the filename.
-  //   
+  //
   //   Contains comment: 'If the [file]name is "/dev/null", ignore the name
   //   and mark the file as being nonexistent.  The name "/dev/null" appears
   //   in patches regardless of how NULL_DEVICE is spelled.'  Also detects
@@ -1367,14 +1380,14 @@ static void dump_incorrect_merge(vector<string> const & expected,
 }
 
 // high tech randomizing test
-
-static void randomizing_merge_test()
+UNIT_TEST(diff_patch, randomizing_merge)
 {
+  randomizer rng;
   for (int i = 0; i < 30; ++i)
     {
       vector<string> anc, d1, d2, m1, m2, gm;
 
-      file_randomizer::build_random_fork(anc, d1, d2, gm, (10 + 2 * i));
+      file_randomizer::build_random_fork(anc, d1, d2, gm, (10 + 2 * i), rng);
 
       BOOST_CHECK(merge3(anc, d1, d2, m1));
       if (gm != m1)
@@ -1390,8 +1403,7 @@ static void randomizing_merge_test()
 
 
 // old boring tests
-
-static void merge_prepend_test()
+UNIT_TEST(diff_patch, merge_prepend)
 {
   BOOST_CHECKPOINT("prepend test");
   vector<string> anc, d1, d2, m1, m2, gm;
@@ -1421,8 +1433,7 @@ static void merge_prepend_test()
   BOOST_CHECK(gm == m2);
 }
 
-
-static void merge_append_test()
+UNIT_TEST(diff_patch, merge_append)
 {
   BOOST_CHECKPOINT("append test");
   vector<string> anc, d1, d2, m1, m2, gm;
@@ -1452,7 +1463,7 @@ static void merge_append_test()
 
 }
 
-static void merge_additions_test()
+UNIT_TEST(diff_patch, merge_additions)
 {
   BOOST_CHECKPOINT("additions test");
   string ancestor("I like oatmeal\nI like orange juice\nI like toast");
@@ -1481,7 +1492,7 @@ static void merge_additions_test()
   BOOST_CHECK(!merge3(anc, d1, cf, m1));
 }
 
-static void merge_deletions_test()
+UNIT_TEST(diff_patch, merge_deletions)
 {
   string ancestor("I like oatmeal\nI like orange juice\nI like toast");
   string desc2("I like oatmeal\nI like toast");
@@ -1504,18 +1515,6 @@ static void merge_deletions_test()
   BOOST_CHECK(gm == m2);
 }
 
-
-void add_diff_patch_tests(test_suite * suite)
-{
-  I(suite);
-  suite->add(BOOST_TEST_CASE(&merge_prepend_test));
-  suite->add(BOOST_TEST_CASE(&merge_append_test));
-  suite->add(BOOST_TEST_CASE(&merge_additions_test));
-  suite->add(BOOST_TEST_CASE(&merge_deletions_test));
-  suite->add(BOOST_TEST_CASE(&randomizing_merge_test));
-}
-
-
 #endif // BUILD_UNIT_TESTS
 
 // Local Variables:
