@@ -9,11 +9,9 @@
 
 #include "config.h"
 
-extern "C" {
-#include <lua.h>
-#include <lualib.h>
-#include <lauxlib.h>
-}
+#include "lua.h"
+#include "lualib.h"
+#include "lauxlib.h"
 
 #include <boost/lexical_cast.hpp>
 #include <boost/filesystem/path.hpp>
@@ -69,7 +67,7 @@ extern "C"
     map<lua_State*, app_state*>::iterator i = map_of_lua_to_app.find(L);
     if (i != map_of_lua_to_app.end())
       {
-        system_path dir = i->second->get_confdir();
+        system_path dir = i->second->opts.conf_dir;
         string confdir = dir.as_external();
         lua_pushstring(L, confdir.c_str());
       }
@@ -81,17 +79,12 @@ extern "C"
 
 lua_hooks::lua_hooks()
 {
-  st = lua_open ();
+  st = luaL_newstate();
   I(st);
 
   lua_atpanic (st, &panic_thrower);
 
-  luaopen_base(st);
-  luaopen_io(st);
-  luaopen_string(st);
-  luaopen_math(st);
-  luaopen_table(st);
-  luaopen_debug(st);
+  luaL_openlibs(st);
 
   lua_register(st, "get_confdir", monotone_get_confdir_for_lua);
   add_functions(st);
@@ -143,7 +136,7 @@ lua_hooks::default_rcfilename(system_path & file)
 {
   map<lua_State*, app_state*>::iterator i = map_of_lua_to_app.find(st);
   I(i != map_of_lua_to_app.end());
-  file = i->second->get_confdir() / "monotonerc";
+  file = i->second->opts.conf_dir / "monotonerc";
 }
 
 void
@@ -288,17 +281,20 @@ lua_hooks::hook_get_author(cert_value const & branchname,
 }
 
 bool
-lua_hooks::hook_edit_comment(string const & commentary,
-                             string const & user_log_message,
-                             string & result)
+lua_hooks::hook_edit_comment(external const & commentary,
+                             external const & user_log_message,
+                             external & result)
 {
-  return Lua(st)
-    .func("edit_comment")
-    .push_str(commentary)
-    .push_str(user_log_message)
-    .call(2,1)
-    .extract_str(result)
-    .ok();
+  string result_str;
+  bool is_ok = Lua(st)
+                 .func("edit_comment")
+                 .push_str(commentary())
+                 .push_str(user_log_message())
+                 .call(2,1)
+                 .extract_str(result_str)
+                 .ok();
+  result = result_str;
+  return is_ok;
 }
 
 bool
@@ -790,17 +786,19 @@ lua_hooks::hook_get_linesep_conv(file_path const & p,
 }
 
 bool
-lua_hooks::hook_validate_commit_message(string const & message,
-                                        string const & new_manifest_text,
+lua_hooks::hook_validate_commit_message(utf8 const & message,
+                                        revision_data const & new_rev,
+                                        cert_value const & branchname,
                                         bool & validated,
                                         string & reason)
 {
   validated = true;
   return Lua(st)
     .func("validate_commit_message")
-    .push_str(message)
-    .push_str(new_manifest_text)
-    .call(2, 2)
+    .push_str(message())
+    .push_str(new_rev.inner()())
+    .push_str(branchname())
+    .call(3, 2)
     .extract_str(reason)
     // XXX When validated, the extra returned string is superfluous.
     .pop()
