@@ -481,8 +481,8 @@ session::session(protocol_role role,
   remote_peer_key_hash(""),
   remote_peer_key_name(""),
   session_key(constants::netsync_key_initializer),
-  read_hmac(constants::netsync_key_initializer, app.use_transport_auth),
-  write_hmac(constants::netsync_key_initializer, app.use_transport_auth),
+  read_hmac(constants::netsync_key_initializer, app.opts.use_transport_auth),
+  write_hmac(constants::netsync_key_initializer, app.opts.use_transport_auth),
   authenticated(false),
   last_io_time(::time(NULL)),
   byte_in_ticker(NULL),
@@ -706,12 +706,12 @@ session::set_session_key(string const & key)
 void
 session::set_session_key(rsa_oaep_sha_data const & hmac_key_encrypted)
 {
-  if (app.use_transport_auth)
+  if (app.opts.use_transport_auth)
     {
       keypair our_kp;
-      load_key_pair(app, app.signing_key, our_kp);
+      load_key_pair(app, app.opts.signing_key, our_kp);
       string hmac_key;
-      decrypt_rsa(app.lua, app.signing_key, our_kp.priv,
+      decrypt_rsa(app.lua, app.opts.signing_key, our_kp.priv,
                   hmac_key_encrypted, hmac_key);
       set_session_key(hmac_key);
     }
@@ -1060,7 +1060,7 @@ session::queue_hello_cmd(rsa_keypair_id const & key_name,
                          id const & nonce)
 {
   rsa_pub_key pub;
-  if (app.use_transport_auth)
+  if (app.opts.use_transport_auth)
     decode_base64(pub_encoded, pub);
   cmd.write_hello_cmd(key_name, pub, nonce);
   write_netcmd_and_try_flush(cmd);
@@ -1075,7 +1075,7 @@ session::queue_anonymous_cmd(protocol_role role,
 {
   netcmd cmd;
   rsa_oaep_sha_data hmac_key_encrypted;
-  if (app.use_transport_auth)
+  if (app.opts.use_transport_auth)
     encrypt_rsa(app.lua, remote_peer_key_name, server_key_encoded,
                 nonce2(), hmac_key_encrypted);
   cmd.write_anonymous_cmd(role, include_pattern, exclude_pattern,
@@ -1096,7 +1096,7 @@ session::queue_auth_cmd(protocol_role role,
 {
   netcmd cmd;
   rsa_oaep_sha_data hmac_key_encrypted;
-  I(app.use_transport_auth);
+  I(app.opts.use_transport_auth);
   encrypt_rsa(app.lua, remote_peer_key_name, server_key_encoded,
               nonce2(), hmac_key_encrypted);
   cmd.write_auth_cmd(role, include_pattern, exclude_pattern, client,
@@ -1220,7 +1220,7 @@ session::process_hello_cmd(rsa_keypair_id const & their_keyname,
 
   base64<rsa_pub_key> their_key_encoded;
 
-  if (app.use_transport_auth)
+  if (app.opts.use_transport_auth)
     {
       hexenc<id> their_key_hash;
       encode_base64(their_key, their_key_encoded);
@@ -1242,7 +1242,7 @@ session::process_hello_cmd(rsa_keypair_id const & their_keyname,
                   "I expected %s\n"
                   "'%s unset %s %s' overrides this check")
                 % their_key_hash % expected_key_hash
-                % app.prog_name % their_key_key.first % their_key_key.second);
+                % ui.prog_name % their_key_key.first % their_key_key.second);
               E(false, F("server key changed"));
             }
         }
@@ -1250,7 +1250,7 @@ session::process_hello_cmd(rsa_keypair_id const & their_keyname,
         {
           P(F("first time connecting to server %s\n"
               "I'll assume it's really them, but you might want to double-check\n"
-              "their key's fingerprint: %s\n") % peer_id % their_key_hash);
+              "their key's fingerprint: %s") % peer_id % their_key_hash);
           app.db.set_var(their_key_key, var_value(their_key_hash()));
         }
       if (!app.db.public_key_exists(their_key_hash))
@@ -1288,23 +1288,23 @@ session::process_hello_cmd(rsa_keypair_id const & their_keyname,
 
   setup_client_tickers();
 
-  if (app.use_transport_auth &&
-      app.signing_key() != "")
+  if (app.opts.use_transport_auth &&
+      app.opts.signing_key() != "")
     {
       // get our key pair
       keypair our_kp;
-      load_key_pair(app, app.signing_key, our_kp);
+      load_key_pair(app, app.opts.signing_key, our_kp);
 
       // get the hash identifier for our pubkey
       hexenc<id> our_key_hash;
       id our_key_hash_raw;
-      key_hash_code(app.signing_key, our_kp.pub, our_key_hash);
+      key_hash_code(app.opts.signing_key, our_kp.pub, our_key_hash);
       decode_hexenc(our_key_hash, our_key_hash_raw);
 
       // make a signature
       base64<rsa_sha1_signature> sig;
       rsa_sha1_signature sig_raw;
-      make_signature(app, app.signing_key, our_kp.priv, nonce(), sig);
+      make_signature(app, app.opts.signing_key, our_kp.priv, nonce(), sig);
       decode_base64(sig, sig_raw);
 
       // make a new nonce of our own and send off the 'auth'
@@ -1344,7 +1344,7 @@ session::process_anonymous_cmd(protocol_role their_role,
   // If running in no-transport-auth mode, we operate anonymously and
   // permit adoption of any role.
 
-  if (app.use_transport_auth)
+  if (app.opts.use_transport_auth)
     {
       if (their_role != sink_role)
         {
@@ -1371,7 +1371,7 @@ session::process_anonymous_cmd(protocol_role their_role,
           {
             error((F("not serving branch '%s'") % *i).str());
           }
-        else if (app.use_transport_auth &&
+        else if (app.opts.use_transport_auth &&
                  !app.lua.hook_get_netsync_read_permitted(*i))
           {
             error((F("anonymous access to branch '%s' denied by server") % *i).str());
@@ -1380,7 +1380,7 @@ session::process_anonymous_cmd(protocol_role their_role,
           ok_branches.insert(utf8(*i));
     }
 
-  if (app.use_transport_auth)
+  if (app.opts.use_transport_auth)
     {
       P(F("allowed anonymous read permission for '%s' excluding '%s'")
         % their_include_pattern % their_exclude_pattern);
@@ -1820,6 +1820,8 @@ session::process_data_cmd(netcmd_item_type type,
   if (data_exists(type, item))
     {
       L(FL("%s '%s' already exists in our database") % typestr % hitem);
+      if (type == epoch_item)
+        maybe_note_epochs_finished();
       return true;
     }
 
@@ -2160,9 +2162,9 @@ void
 session::begin_service()
 {
   keypair kp;
-  if (app.use_transport_auth)
-    app.keys.get_key_pair(app.signing_key, kp);
-  queue_hello_cmd(app.signing_key, kp.pub, mk_nonce());
+  if (app.opts.use_transport_auth)
+    app.keys.get_key_pair(app.opts.signing_key, kp);
+  queue_hello_cmd(app.opts.signing_key, kp.pub, mk_nonce());
 }
 
 void
@@ -2224,7 +2226,7 @@ bool session::process(transaction_guard & guard)
 
       if (inbuf.size() >= constants::netcmd_maxsz)
         W(F("input buffer for peer %s is overfull "
-            "after netcmd dispatch\n") % peer_id);
+            "after netcmd dispatch") % peer_id);
 
       guard.maybe_checkpoint(sz);
 
@@ -2270,7 +2272,7 @@ build_stream_to_server(app_state & app,
       I(argv.size() > 0);
       string cmd = argv[0];
       argv.erase(argv.begin());
-      app.use_transport_auth = app.lua.hook_use_transport_auth(u);
+      app.opts.use_transport_auth = app.lua.hook_use_transport_auth(u);
       return shared_ptr<Netxx::StreamBase>
         (new Netxx::PipeStream(cmd, argv));
 
@@ -2350,7 +2352,7 @@ call_server(protocol_role role,
       if (fd == -1 && !armed)
         {
           E(false, (F("timed out waiting for I/O with "
-                      "peer %s, disconnecting\n")
+                      "peer %s, disconnecting")
                     % sess.peer_id));
         }
 
@@ -2373,7 +2375,7 @@ call_server(protocol_role role,
             // error from our server (which is translated to a decode
             // exception). We call these cases E() errors.
             E(false, F("processing failure while talking to "
-                       "peer %s, disconnecting\n")
+                       "peer %s, disconnecting")
               % sess.peer_id);
             return;
           }
@@ -2401,7 +2403,7 @@ call_server(protocol_role role,
             }
           else
             E(false, (F("I/O failure while talking to "
-                        "peer %s, disconnecting\n")
+                        "peer %s, disconnecting")
                       % sess.peer_id));
         }
     }
@@ -2552,7 +2554,7 @@ handle_read_available(Netxx::socket_type fd,
 
         case session::shutdown_state:
           P(F("peer %s read failed in shutdown state "
-              "(possibly client misreported error)\n")
+              "(possibly client misreported error)")
             % sess->peer_id);
           break;
 
@@ -2584,7 +2586,7 @@ handle_write_available(Netxx::socket_type fd,
 
         case session::shutdown_state:
           P(F("peer %s write failed in shutdown state "
-              "(possibly client misreported error)\n")
+              "(possibly client misreported error)")
             % sess->peer_id);
           break;
 
@@ -2667,8 +2669,8 @@ serve_connections(protocol_role role,
     timeout(static_cast<long>(timeout_seconds)),
     instant(0,1);
 
-  if (!app.bind_port().empty())
-    default_port = std::atoi(app.bind_port().c_str());
+  if (!app.opts.bind_port().empty())
+    default_port = std::atoi(app.opts.bind_port().c_str());
 #ifdef USE_IPV6
   bool use_ipv6=true;
 #else
@@ -2686,8 +2688,8 @@ serve_connections(protocol_role role,
 
           Netxx::Address addr(use_ipv6);
 
-          if (!app.bind_address().empty())
-            addr.add_address(app.bind_address().c_str(), default_port);
+          if (!app.opts.bind_address().empty())
+            addr.add_address(app.opts.bind_address().c_str(), default_port);
           else
             addr.add_all_addresses (default_port);
 
@@ -2722,7 +2724,7 @@ serve_connections(protocol_role role,
 
               if (sessions.size() >= session_limit)
                 W(F("session limit %d reached, some connections "
-                    "will be refused\n") % session_limit);
+                    "will be refused") % session_limit);
               else
                 probe.add(server);
 
@@ -3069,8 +3071,8 @@ session::rebuild_merkle_trees(app_state & app,
 
   // Add any keys specified on the command line.
   for (vector<rsa_keypair_id>::const_iterator key
-         = app.keys_to_push.begin();
-       key != app.keys_to_push.end(); ++key)
+         = app.opts.keys_to_push.begin();
+       key != app.opts.keys_to_push.end(); ++key)
     {
       if (inserted_keys.find(*key) == inserted_keys.end())
         {
@@ -3120,20 +3122,23 @@ run_netsync_protocol(protocol_voice voice,
   if (include_pattern().find_first_of("'\"") != string::npos)
     {
       W(F("include branch pattern contains a quote character:\n"
-          "%s\n") % include_pattern());
+          "%s") % include_pattern());
     }
 
   if (exclude_pattern().find_first_of("'\"") != string::npos)
     {
       W(F("exclude branch pattern contains a quote character:\n"
-          "%s\n") % exclude_pattern());
+          "%s") % exclude_pattern());
     }
 
+  // We do not want to be killed by SIGPIPE from a network disconnect.
+  ignore_sigpipe();
+
   try
     {
       if (voice == server_voice)
         {
-          if (app.bind_stdio)
+          if (app.opts.bind_stdio)
             {
               shared_ptr<Netxx::PipeStream> str(new Netxx::PipeStream(0,1));
               shared_ptr<session> sess(new session(role, server_voice,
