@@ -35,7 +35,7 @@
 #include "keys.hh"
 #include "numeric_vocab.hh"
 #include "revision.hh"
-#include "sanity.hh"
+#include "mtn-sanity.hh"
 #include "transforms.hh"
 #include "simplestring_xform.hh"
 #include "ui.hh"
@@ -116,10 +116,12 @@ revision_t::is_nontrivial() const
 revision_t::revision_t(revision_t const & other)
 {
   /* behave like normal constructor if other is empty */
+  made_for = made_for_nobody;
   if (null_id(other.new_manifest) && other.edges.empty()) return;
   other.check_sane();
   new_manifest = other.new_manifest;
   edges = other.edges;
+  made_for = other.made_for;
 }
 
 revision_t const &
@@ -128,6 +130,7 @@ revision_t::operator=(revision_t const & other)
   other.check_sane();
   new_manifest = other.new_manifest;
   edges = other.edges;
+  made_for = other.made_for;
   return *this;
 }
 
@@ -430,42 +433,117 @@ toposort(set<revision_id> const & revisions,
     }
 }
 
+static void
+accumulate_strict_ancestors(revision_id const & start,
+                            set<revision_id> & new_ancestors,
+                            set<revision_id> & all_ancestors,
+                            multimap<revision_id, revision_id> const & inverse_graph)
+{
+  typedef multimap<revision_id, revision_id>::const_iterator gi;
+  new_ancestors.clear();
+
+  vector<revision_id> frontier;
+  frontier.push_back(start);
+  while (!frontier.empty())
+    {
+      revision_id rid = frontier.back();
+      frontier.pop_back();
+      pair<gi, gi> parents = inverse_graph.equal_range(rid);
+      for (gi i = parents.first; i != parents.second; ++i)
+        {
+          revision_id const & parent = i->second;
+          if (all_ancestors.find(parent) == all_ancestors.end())
+            {
+              new_ancestors.insert(parent);
+              all_ancestors.insert(parent);
+              frontier.push_back(i->second);
+            }
+        }
+    }
+}
+
+// this call is equivalent to running:
+//   remove_if(candidates.begin(), candidates.end(), p);
+//   erase_ancestors(candidates, app);
+// however, by interleaving the two operations, it can in common cases make
+// many fewer calls to the predicate, which can be a significant speed win.
+//
+// FIXME: once we have heights, we should use them here.  The strategy will
+// be, to calculate the minimum height of all the candidates, and teach
+// accumulate_ancestors to stop at a certain height, and teach the code that
+// removes items from the candidate set to occasionally rescan the candidate
+// set to get a new minimum height (perhaps, whenever we remove the minimum
+// height candidate).
+void
+erase_ancestors_and_failures(std::set<revision_id> & candidates,
+                             is_failure & p,
+                             app_state & app)
+{
+  // Load up the ancestry graph
+  multimap<revision_id, revision_id> inverse_graph;
+  
+  {
+    multimap<revision_id, revision_id> graph;
+    app.db.get_revision_ancestry(graph);
+    for (multimap<revision_id, revision_id>::const_iterator i = graph.begin();
+         i != graph.end(); ++i)
+      inverse_graph.insert(make_pair(i->second, i->first));
+  }
+
+  // Keep a set of all ancestors that we've traversed -- to avoid
+  // combinatorial explosion.
+  set<revision_id> all_ancestors;
+
+  vector<revision_id> todo(candidates.begin(), candidates.end());
+  std::random_shuffle(todo.begin(), todo.end());
+
+  size_t predicates = 0;
+  while (!todo.empty())
+    {
+      revision_id rid = todo.back();
+      todo.pop_back();
+      // check if this one has already been eliminated
+      if (all_ancestors.find(rid) != all_ancestors.end())
+        continue;
+      // and then whether it actually should stay in the running:
+      ++predicates;
+      if (p(rid))
+        {
+          candidates.erase(rid);
+          continue;
+        }
+      // okay, it is good enough that all its ancestors should be
+      // eliminated; do that now.
+      {
+        set<revision_id> new_ancestors;
+        accumulate_strict_ancestors(rid, new_ancestors, all_ancestors, inverse_graph);
+        I(new_ancestors.find(rid) == new_ancestors.end());
+        for (set<revision_id>::const_iterator i = new_ancestors.begin();
+             i != new_ancestors.end(); ++i)
+          candidates.erase(*i);
+      }
+    }
+  L(FL("called predicate %s times") % predicates);
+}
+
 // This function looks at a set of revisions, and for every pair A, B in that
 // set such that A is an ancestor of B, it erases A.
 
+namespace
+{
+  struct no_failures : public is_failure
+  {
+    virtual bool operator()(revision_id const & rid)
+    {
+      return false;
+    }
+  };
+}
 void
 erase_ancestors(set<revision_id> & revisions, app_state & app)
 {
-  typedef multimap<revision_id, revision_id>::const_iterator gi;
-  multimap<revision_id, revision_id> graph;
-  multimap<revision_id, revision_id> inverse_graph;
-
-  app.db.get_revision_ancestry(graph);
-  for (gi i = graph.begin(); i != graph.end(); ++i)
-    inverse_graph.insert(make_pair(i->second, i->first));
-
-  interner<ctx> intern;
-  map< ctx, shared_bitmap > ancestors;
-
-  shared_bitmap u = shared_bitmap(new bitmap());
-
-  for (set<revision_id>::const_iterator i = revisions.begin();
-       i != revisions.end(); ++i)
-    {
-      calculate_ancestors_from_graph(intern, *i, inverse_graph, ancestors, u);
-    }
-
-  set<revision_id> tmp;
-  for (set<revision_id>::const_iterator i = revisions.begin();
-       i != revisions.end(); ++i)
-    {
-      ctx id = intern.intern(i->inner()());
-      bool has_ancestor_in_set = id < u->size() && u->test(id);
-      if (!has_ancestor_in_set)
-        tmp.insert(*i);
-    }
-
-  revisions = tmp;
+  no_failures p;
+  erase_ancestors_and_failures(revisions, p, app);
 }
 
 // This function takes a revision A and a set of revision Bs, calculates the
@@ -528,8 +606,7 @@ ancestry_difference(revision_id const & a, set<revision_id> const & bs,
 }
 
 void
-select_nodes_modified_by_rev(revision_id const & rid,
-                             revision_t const & rev,
+select_nodes_modified_by_rev(revision_t const & rev,
                              roster_t const new_roster,
                              set<node_id> & nodes_modified,
                              app_state & app)
@@ -568,8 +645,66 @@ make_revision(revision_id const & old_rev_id,
   L(FL("new manifest_id is %s") % rev.new_manifest);
 
   safe_insert(rev.edges, make_pair(old_rev_id, cs));
+  rev.made_for = made_for_database;
 }
 
+void
+make_revision(revision_id const & old_rev_id,
+              roster_t const & old_roster,
+              cset const & changes,
+              revision_t & rev)
+{
+  roster_t new_roster = old_roster;
+  {
+    temp_node_id_source nis;
+    editable_roster_base er(new_roster, nis);
+    changes.apply_to(er);
+  }
+
+  shared_ptr<cset> cs(new cset(changes));
+  rev.edges.clear();
+
+  calculate_ident(new_roster, rev.new_manifest);
+  L(FL("new manifest_id is %s") % rev.new_manifest);
+
+  safe_insert(rev.edges, make_pair(old_rev_id, cs));
+  rev.made_for = made_for_database;
+}
+
+// Workspace-only revisions, with fake rev.new_manifest and content
+// changes suppressed.
+void
+make_revision_for_workspace(revision_id const & old_rev_id,
+                            cset const & changes,
+                            revision_t & rev)
+{
+  MM(old_rev_id);
+  MM(changes);
+  MM(rev);
+  shared_ptr<cset> cs(new cset(changes));
+  cs->deltas_applied.clear();
+
+  rev.edges.clear();
+  safe_insert(rev.edges, make_pair(old_rev_id, cs));
+  if (!null_id(old_rev_id))
+    rev.new_manifest = fake_id();
+  rev.made_for = made_for_workspace;
+}
+
+void
+make_revision_for_workspace(revision_id const & old_rev_id,
+                            roster_t const & old_roster,
+                            roster_t const & new_roster,
+                            revision_t & rev)
+{
+  MM(old_rev_id);
+  MM(old_roster);
+  MM(new_roster);
+  MM(rev);
+  cset changes;
+  make_cset(old_roster, new_roster, changes);
+  make_revision_for_workspace(old_rev_id, changes, rev);
+}
 
 // Stuff related to rebuilding the revision graph. Unfortunately this is a
 // real enough error case that we need support code for it.
@@ -994,7 +1129,7 @@ insert_into_roster(roster_t & child_roster,
   pth.split(sp);
 
   E(!child_roster.has_node(sp),
-    F("Path %s added to child roster multiple times\n") % pth);
+    F("Path %s added to child roster multiple times") % pth);
 
   dirname_basename(sp, dirname, basename);
 
@@ -1007,7 +1142,7 @@ insert_into_roster(roster_t & child_roster,
         if (child_roster.has_node(tmp_pth))
           {
             E(is_dir_t(child_roster.get_node(tmp_pth)),
-              F("Directory for path %s cannot be added, as there is a file in the way\n") % pth);
+              F("Directory for path %s cannot be added, as there is a file in the way") % pth);
           }
         else
           child_roster.attach_node(child_roster.create_dir_node(nis), tmp_pth);
@@ -1021,7 +1156,7 @@ insert_into_roster(roster_t & child_roster,
         F("Path %s cannot be added, as there is a directory in the way") % sp);
       file_t f = downcast_to_file_t(n);
       E(f->content == fid,
-        F("Path %s added twice with differing content\n") % sp);
+        F("Path %s added twice with differing content") % sp);
     }
   else
     child_roster.attach_node(child_roster.create_file_node(fid, nis), sp);
@@ -1299,7 +1434,7 @@ anc_graph::construct_revisions_from_ancestry()
                              k != fattrs.end(); ++k)
                           {
                             string key = k->first;
-                            if (app.attrs_to_drop.find(key) != app.attrs_to_drop.end())
+                            if (app.opts.attrs_to_drop.find(key) != app.opts.attrs_to_drop.end())
                               {
                                 // ignore it
                               }
@@ -1324,6 +1459,7 @@ anc_graph::construct_revisions_from_ancestry()
           fixup_node_identities(parent_rosters, child_roster, node_to_renames[child]);
 
           revision_t rev;
+          rev.made_for = made_for_database;
           MM(rev);
           calculate_ident(child_roster, rev.new_manifest);
 
@@ -1417,7 +1553,7 @@ build_roster_style_revs_from_manifest_style_revs(app_state & app)
   app.db.ensure_open_for_format_changes();
   app.db.check_is_not_rosterified();
 
-  global_sanity.set_relaxed(true);
+  real_sanity.set_relaxed(true);
   anc_graph graph(true, app);
 
   P(F("converting existing revision graph to new roster-style revisions"));
@@ -1462,7 +1598,7 @@ build_roster_style_revs_from_manifest_style_revs(app_state & app)
       graph.add_node_for_oldstyle_revision(*i);
     }
 
-  global_sanity.set_relaxed(false);
+  real_sanity.set_relaxed(false);
   graph.rebuild_ancestry();
 }
 
@@ -1505,6 +1641,37 @@ build_changesets_from_manifest_ancestry(app_state & app)
   graph.rebuild_ancestry();
 }
 
+void
+regenerate_rosters(app_state & app)
+{
+  P(F("regenerating cached rosters"));
+
+  app.db.ensure_open_for_format_changes();
+  transaction_guard guard(app.db);
+
+  app.db.delete_existing_rosters();
+
+  std::set<revision_id> ids;
+  app.db.get_revision_ids(ids);
+  P(F("calculating rosters to regenerate"));
+  std::vector<revision_id> sorted_ids;
+  toposort(ids, sorted_ids, app);
+
+  ticker done(_("regenerated"), "r", 5);
+  done.set_total(sorted_ids.size());
+
+  for (std::vector<revision_id>::const_iterator i = sorted_ids.begin();
+       i != sorted_ids.end(); ++i)
+    {
+      revision_t rev;
+      app.db.get_revision(*i, rev);
+      app.db.put_roster_for_revision(*i, rev);
+      ++done;
+    }
+
+  guard.commit();
+}
+
 // i/o stuff
 
 namespace
@@ -1580,6 +1747,7 @@ parse_revision(basic_io::parser & parser,
 {
   MM(rev);
   rev.edges.clear();
+  rev.made_for = made_for_database;
   string tmp;
   parser.esym(syms::format_version);
   parser.str(tmp);
@@ -1664,8 +1832,7 @@ void calculate_ident(revision_t const & cs,
 #include "unit_tests.hh"
 #include "sanity.hh"
 
-static void
-test_find_old_new_path_for()
+UNIT_TEST(revision, find_old_new_path_for)
 {
   map<split_path, split_path> renames;
   split_path foo, foo_bar, foo_baz, quux, quux_baz;
@@ -1690,14 +1857,6 @@ test_find_old_new_path_for()
   I(foo_bar == find_new_path_for(renames, foo_baz));
 }
 
-void
-add_revision_tests(test_suite * suite)
-{
-  I(suite);
-  suite->add(BOOST_TEST_CASE(&test_find_old_new_path_for));
-}
-
-
 #endif // BUILD_UNIT_TESTS
 
 // Local Variables:
