@@ -612,6 +612,8 @@ roster_merge(roster_t const & left_parent,
 #ifdef BUILD_UNIT_TESTS
 #include "unit_tests.hh"
 
+#include "roster_delta.hh"
+
 // cases for testing:
 //
 // (DONE:)
@@ -768,6 +770,9 @@ test_a_scalar_merge_impl(scalar_val left_val, string const & left_marks_str,
                right_parent, right_markings, right_uncommon_ancestors,
                result);
 
+  // go ahead and check the roster_delta code too, while we're at it...
+  test_roster_delta_on(left_parent, left_markings, right_parent, right_markings);
+
   scalar.check_result(left_val, right_val, result, expected_outcome);
 }
 
@@ -1075,9 +1080,7 @@ test_a_scalar_merge(scalar_val left_val, string const & left_marks_str,
                                                 expected_outcome);
 }
 
-
-void
-test_scalar_merges()
+UNIT_TEST(roster_merge, scalar_merges)
 {
   // Notation: a1* means, "value is a, this is node 1 in the graph, it is
   // marked".  ".2" means, "value is unimportant and different from either a
@@ -1244,9 +1247,7 @@ make_node_lifecycle_objs(roster_t & r, marking_map & markings, revision_id const
   make_file(r, markings, common1, common1, common1, name + "_dead_file", fid1, nis.next());
 }
 
-
-static void
-test_roster_merge_node_lifecycle()
+UNIT_TEST(roster_merge, node_lifecycle)
 {
   roster_t a_roster, b_roster;
   marking_map a_markings, b_markings;
@@ -1275,6 +1276,8 @@ test_roster_merge_node_lifecycle()
   roster_merge_result result;
   roster_merge(a_roster, a_markings, a_uncommon, b_roster, b_markings, b_uncommon, result);
   I(result.is_clean());
+  // go ahead and check the roster_delta code too, while we're at it...
+  test_roster_delta_on(a_roster, a_markings, b_roster, b_markings);
   // 7 = 1 root + 2 common + 2 safe a + 2 safe b
   I(result.roster.all_nodes().size() == 7);
   // check that they're the right ones...
@@ -1296,8 +1299,7 @@ test_roster_merge_node_lifecycle()
                   b_roster.get_node(b_safe_file_nid), false));
 }
 
-static void
-test_roster_merge_attr_lifecycle()
+UNIT_TEST(roster_merge, attr_lifecycle)
 {
   roster_t left_roster, right_roster;
   marking_map left_markings, right_markings;
@@ -1351,6 +1353,8 @@ test_roster_merge_attr_lifecycle()
   roster_merge(left_roster, left_markings, left_revs,
                right_roster, right_markings, right_revs,
                result);
+  // go ahead and check the roster_delta code too, while we're at it...
+  test_roster_delta_on(left_roster, left_markings, right_roster, right_markings);
   I(result.roster.all_nodes().size() == 2);
   I(result.roster.get_node(dir_nid)->attrs.size() == 4);
   I(safe_get(result.roster.get_node(dir_nid)->attrs, attr_key("left_live")) == make_pair(true, attr_value("left_live")));
@@ -1399,6 +1403,8 @@ struct structural_conflict_helper
     roster_merge(left_roster, left_markings, left_revs,
                  right_roster, right_markings, right_revs,
                  result);
+    // go ahead and check the roster_delta code too, while we're at it...
+    test_roster_delta_on(left_roster, left_markings, right_roster, right_markings);
 
     check();
   }
@@ -1581,9 +1587,7 @@ struct simple_missing_root_dir : public structural_conflict_helper
     }
 };
 
-
-static void
-test_simple_structural_conflicts()
+UNIT_TEST(roster_merge, simple_structural_conflicts)
 {
   {
     simple_rename_target_conflict t;
@@ -1815,8 +1819,7 @@ struct rename_target_plus_missing_root : public structural_conflict_helper
   }
 };
 
-static void
-test_complex_structural_conflicts()
+UNIT_TEST(roster_merge, complex_structural_conflicts)
 {
   {
     node_name_plus_rename_target t;
@@ -1844,17 +1847,6 @@ test_complex_structural_conflicts()
   }
 }
 
-void
-add_roster_merge_tests(test_suite * suite)
-{
-  I(suite);
-  suite->add(BOOST_TEST_CASE(&test_roster_merge_node_lifecycle));
-  suite->add(BOOST_TEST_CASE(&test_roster_merge_attr_lifecycle));
-  suite->add(BOOST_TEST_CASE(&test_scalar_merges));
-  suite->add(BOOST_TEST_CASE(&test_simple_structural_conflicts));
-  suite->add(BOOST_TEST_CASE(&test_complex_structural_conflicts));
-}
-
 #endif // BUILD_UNIT_TESTS
 
 // Local Variables:
