@@ -25,6 +25,7 @@
 #include "app_state.hh"
 #include "keys.hh"
 #include "transforms.hh"
+#include "ui.hh"
 
 using std::ctype;
 using std::locale;
@@ -208,7 +209,23 @@ calculate_schema_id(sqlite3 *sql, string & id)
   calculate_id(tmp2, id);
 }
 
-typedef bool (*migrator_cb)(sqlite3 *, char **, app_state *);
+// these must be listed in order so that ones listed earlier override ones
+// listed later
+enum upgrade_regime
+  {
+    upgrade_changesetify,
+    upgrade_rosterify,
+    upgrade_regen_rosters,
+    upgrade_none, 
+  };
+
+static void
+set_regime(upgrade_regime new_regime, upgrade_regime & regime)
+{
+  regime = std::min(new_regime, regime);
+}
+
+typedef bool (*migrator_cb)(sqlite3 *, char **, app_state *, upgrade_regime &);
 
 struct
 migrator
@@ -240,6 +257,8 @@ migrator
 
     P(F("calculating necessary migration steps"));
 
+    upgrade_regime regime = upgrade_none;
+
     bool migrating = false;
     for (vector< pair<string, migrator_cb> >::const_iterator i = migration_events.begin();
          i != migration_events.end(); ++i)
@@ -272,7 +291,7 @@ migrator
               }
 
             // do this migration step
-            else if (! i->second(sql, &errmsg, __app))
+            else if (! i->second(sql, &errmsg, __app, regime))
               {
                 logged_sqlite3_exec(sql, "ROLLBACK", NULL, NULL, NULL);
                 E(false, F("migration step failed: %s")
@@ -302,6 +321,28 @@ migrator
 
         E(logged_sqlite3_exec(sql, "ANALYZE", NULL, NULL, NULL) == SQLITE_OK,
           F("error running analyze after migration"));
+
+        switch (regime)
+          {
+          case upgrade_changesetify:
+          case upgrade_rosterify:
+            {
+              string command_str = (regime == upgrade_changesetify
+                                    ? "changesetify" : "rosterify");
+              P(F("NOTE: because this database was last used by a rather old version\n"
+                  "of monotone, you're not done yet.  If you're a project leader, then\n"
+                  "see the file UPGRADE for instructions on running '%s db %s'")
+                % ui.prog_name % command_str);
+            }
+            break;
+          case upgrade_regen_rosters:
+            P(F("NOTE: this upgrade cleared the roster cache\n"
+                "you should now run '%s db regenerate_rosters'")
+              % ui.prog_name);
+            break;
+          case upgrade_none:
+            break;
+          }
       }
     else
       {
@@ -355,7 +396,8 @@ static bool move_table(sqlite3 *sql, char **errmsg,
 static bool
 migrate_client_merge_url_and_group(sqlite3 * sql,
                                    char ** errmsg,
-                                   app_state *)
+                                   app_state *,
+                                   upgrade_regime & regime)
 {
 
   // migrate the posting_queue table
@@ -499,7 +541,8 @@ migrate_client_merge_url_and_group(sqlite3 * sql,
 static bool
 migrate_client_add_hashes_and_merkle_trees(sqlite3 * sql,
                                            char ** errmsg,
-                                           app_state *)
+                                           app_state *,
+                                           upgrade_regime & regime)
 {
 
   // add the column to manifest_certs
@@ -663,7 +706,8 @@ migrate_client_add_hashes_and_merkle_trees(sqlite3 * sql,
 static bool
 migrate_client_to_revisions(sqlite3 * sql,
                             char ** errmsg,
-                            app_state *)
+                            app_state *,
+                            upgrade_regime & regime)
 {
   int res;
 
@@ -738,6 +782,8 @@ migrate_client_to_revisions(sqlite3 * sql,
   if (res != SQLITE_OK)
     return false;
 
+  set_regime(upgrade_changesetify, regime);
+
   return true;
 }
 
@@ -745,7 +791,8 @@ migrate_client_to_revisions(sqlite3 * sql,
 static bool
 migrate_client_to_epochs(sqlite3 * sql,
                          char ** errmsg,
-                         app_state *)
+                         app_state *,
+                         upgrade_regime & regime)
 {
   int res;
 
@@ -770,7 +817,8 @@ migrate_client_to_epochs(sqlite3 * sql,
 static bool
 migrate_client_to_vars(sqlite3 * sql,
                        char ** errmsg,
-                       app_state *)
+                       app_state *,
+                       upgrade_regime & regime)
 {
   int res;
 
@@ -791,7 +839,8 @@ migrate_client_to_vars(sqlite3 * sql,
 static bool
 migrate_client_to_add_indexes(sqlite3 * sql,
                               char ** errmsg,
-                              app_state *)
+                              app_state *,
+                              upgrade_regime & regime)
 {
   int res;
 
@@ -831,7 +880,8 @@ extract_key(void *ptr, int ncols, char **values, char **names)
 static bool
 migrate_client_to_external_privkeys(sqlite3 * sql,
                                     char ** errmsg,
-                                    app_state *app)
+                                    app_state *app,
+                                    upgrade_regime & regime)
 {
   int res;
   map<string, string> pub, priv;
@@ -881,7 +931,8 @@ migrate_client_to_external_privkeys(sqlite3 * sql,
 static bool
 migrate_client_to_add_rosters(sqlite3 * sql,
                               char ** errmsg,
-                              app_state *)
+                              app_state *,
+                              upgrade_regime & regime)
 {
   int res;
 
@@ -926,6 +977,8 @@ migrate_client_to_add_rosters(sqlite3 * sql,
   if (res != SQLITE_OK)
     return false;
 
+  set_regime(upgrade_rosterify, regime);
+
   return true;
 }
 
@@ -945,8 +998,9 @@ sqlite3_unbase64_fn(sqlite3_context *f, int nargs, sqlite3_value ** args)
 // I wish I had a form of ALTER TABLE COMMENT on sqlite3
 static bool
 migrate_files_BLOB(sqlite3 * sql,
-                                    char ** errmsg,
-                                    app_state *app)
+                   char ** errmsg,
+                   app_state *app,
+                   upgrade_regime & regime)
 {
   int res;
   I(sqlite3_create_function(sql, "unbase64", -1,
@@ -1058,6 +1112,53 @@ migrate_files_BLOB(sqlite3 * sql,
   return true;
 }
 
+static bool
+migrate_rosters_no_hash(sqlite3 * sql,
+                        char ** errmsg,
+                        app_state * app,
+                        upgrade_regime & regime)
+{
+  int res;
+  
+  res = logged_sqlite3_exec(sql, "DROP TABLE rosters", NULL, NULL, errmsg);
+  if (res != SQLITE_OK)
+    return false;
+  res = logged_sqlite3_exec(sql, "DROP TABLE roster_deltas", NULL, NULL, errmsg);
+  if (res != SQLITE_OK)
+    return false;
+  res = logged_sqlite3_exec(sql, "DROP TABLE revision_roster", NULL, NULL, errmsg);
+  if (res != SQLITE_OK)
+    return false;
+
+  res = logged_sqlite3_exec(sql,
+                            "CREATE TABLE rosters\n"
+                            "\t(\n"
+                            "\tid primary key,         -- a revision id\n"
+                            "\tchecksum not null,      -- checksum of 'data', to protect against disk corruption\n"
+                            "\tdata not null           -- compressed, encoded contents of the roster\n"
+                            "\t);",
+                            NULL, NULL, errmsg);
+  if (res != SQLITE_OK)
+    return false;
+
+  res = logged_sqlite3_exec(sql,
+                            "CREATE TABLE roster_deltas\n"
+                            "\t(\n"
+                            "\tid primary key,         -- a revision id\n"
+                            "\tchecksum not null,      -- checksum of 'delta', to protect against disk corruption\n"
+                            "\tbase not null,          -- joins with either rosters.id or roster_deltas.id\n"
+                            "\tdelta not null          -- rdiff to construct current from base\n"
+                            ");",
+                            NULL, NULL, errmsg);
+  if (res != SQLITE_OK)
+    return false;
+
+  set_regime(upgrade_regen_rosters, regime);
+
+  return true;
+}
+
+
 void
 migrate_monotone_schema(sqlite3 *sql, app_state *app)
 {
@@ -1092,11 +1193,14 @@ migrate_monotone_schema(sqlite3 *sql, app_state *app)
   m.add("1db80c7cee8fa966913db1a463ed50bf1b0e5b0e",
         &migrate_files_BLOB);
 
+  m.add("9d2b5d7b86df00c30ac34fe87a3c20f1195bb2df",
+        &migrate_rosters_no_hash);
+
   // IMPORTANT: whenever you modify this to add a new schema version, you must
   // also add a new migration test for the new schema version.  See
   // tests/t_migrate_schema.at for details.
 
-  m.migrate(sql, "9d2b5d7b86df00c30ac34fe87a3c20f1195bb2df");
+  m.migrate(sql, "ae196843d368d042f475e3dadfed11e9d7f9f01e");
 }
 
 // Local Variables:
