@@ -30,11 +30,31 @@ end
 check(mtn("ls", "known"), 0)
 for _,i in pairs(names) do remove(i) end
 
+-- just to make sure, check that it's not only add that fails, if it somehow
+-- sneaks into an internal format then that fails too
+remove("_MTN")
+check(mtn("--branch=testbranch", "setup", "."))
+mkdir("_mTn")
+writefile("_MTN/revision",
+             'format_version "1"\n'
+          .. '\n'
+          .. 'new_manifest []\n'
+          .. '\n'
+          .. 'old_revision []\n'
+          .. '\n'
+          .. 'add_dir ""\n'
+          .. '\n'
+          .. 'add_dir "_mTn"\n')
+check(mtn("status"), 3, false, false)
+check(mtn("commit", "-m", "blah"), 3, false, false)
+
 -- assert trips if we have a db that already has a file with this sort
--- of name in it.
-for _,i in {"files", "dirs"} do
+-- of name in it.  it would be better to test that checkout or pull fail, but
+-- it is too difficult to regenerate this database every time things change,
+-- and in fact we know that the same code paths are exercised by this.
+for _,i in pairs({"files", "dirs"}) do
   get(i..".db.dumped", "stdin")
   check(mtn("db", "load", "-d", i..".mtn"), 0, false, false, true)
   check(mtn("db", "migrate", "-d", i..".mtn"), 0, false, false)
-  check(mtn("-d", i..".mtn", "co", "-b", "testbranch", i.."-co-dir"), 3, false, false)
+  check(mtn("-d", i..".mtn", "db", "regenerate_rosters"), 3, false, false)
 end
