@@ -14,7 +14,7 @@ writefile("foo.C", "c\nident\nx\n")
 -- ident
 -- d
 
-get("merge.lua")
+check(get("merge.lua"))
 
 
 copy("foo.A", "foo")
@@ -50,5 +50,5 @@ L("revs.d = ", revs.d, "\n")
 -- REVD: d
 --
 
-check(mtn("--debug", "annotate", "foo"), 0, true, true)
-greplines("stdout", {revs.a, revs.a, revs.b, revs.c, revs.a, revs.d})
+check(mtn("--debug", "annotate", "--brief", "foo"), 0, true, true)
+check(greplines("stdout", {revs.a, revs.a, revs.b, revs.c, revs.a, revs.d}))
