@@ -6,7 +6,7 @@ writefile("testfile1", "version 1 of second test file\n")
 commit()
 v1 = base_revision()
 addfile("testfile1")
-check(exists("_MTN/work"))
+check(qgrep("add_file","_MTN/revision"))
 remove("_MTN")
 check(mtn("checkout", "--revision", v1, "."), 0, false, false)
-check(not exists("_MTN/work"))
+check(not qgrep("add_file","_MTN/revision"))
