@@ -29,7 +29,7 @@ REV4=base_revision()
 check(mtn("log", "--diffs"), 0, true, false)
 check(grep('^(---|\\+\\+\\+) ', "stdout"), 0, true, false)
 rename("stdout", "full")
-get("expect_full")
+check(get("expect_full"))
 canonicalize("full")
 check(samefile("expect_full", "full"))
 
@@ -37,6 +37,6 @@ check(samefile("expect_full", "full"))
 check(mtn("log", "quux", "--diffs", "foo2"), 0, true, false)
 check(grep("^(---|\\+\\+\\+) ", "stdout"), 0, true, false)
 rename("stdout", "restrict")
-get("expect_restrict")
+check(get("expect_restrict"))
 canonicalize("restrict")
 check(samefile("expect_restrict", "restrict"))
