@@ -29,7 +29,7 @@ check(mtn("log", "baz"), 1, 0, false)
 check(mtn("log", "bar"), 0, true, false)
 rename("stdout", "log")
 check(grep("^Addition of [a-z][a-z][a-z].$", "log"), 0, true)
-get("first")
+check(get("first"))
 canonicalize("first")
 canonicalize("stdout")
 check(samefile("stdout", "first"))
@@ -37,7 +37,7 @@ check(samefile("stdout", "first"))
 check(mtn("log"), 0, true, false)
 rename("stdout", "log")
 check(grep("^Addition of [a-z][a-z][a-z].$", "log"), 0, true)
-get("second")
+check(get("second"))
 canonicalize("second")
 canonicalize("stdout")
 check(samefile("stdout", "second"))
