@@ -9,27 +9,29 @@ commit()
 
 writefile("foofile0", "squirrils monkeys dingos")
 check(mtn("rename", "testfile0", "foofile0"), 0, false, false)
-check(qgrep("testfile0", "_MTN/work"))
-check(qgrep("foofile0", "_MTN/work"))
+check(qgrep("testfile0", "_MTN/revision"))
+check(qgrep("foofile0", "_MTN/revision"))
 check(mtn("status"), 0, true)
 check(qgrep("testfile0", "stdout"))
 check(qgrep("foofile0", "stdout"))
 check(mtn("revert", "foofile0"), 0, false, false)
-check(not exists("_MTN/work"))
+check(not qgrep("testfile0", "_MTN/revision"))
+check(not qgrep("foofile0", "_MTN/revision"))
 check(mtn("status"), 0, true)
 check(not qgrep("testfile0", "stdout"))
 check(not qgrep("foofile0", "stdout"))
 
 writefile("foofile0", "squirrils monkeys dingos")
 check(mtn("rename", "testfile0", "foofile0"), 0, false, false)
-check(qgrep("testfile0", "_MTN/work"))
-check(qgrep("foofile0", "_MTN/work"))
+check(qgrep("testfile0", "_MTN/revision"))
+check(qgrep("foofile0", "_MTN/revision"))
 check(mtn("status"), 0, true)
 check(qgrep("testfile0", "stdout"))
 check(qgrep("foofile0", "stdout"))
 check(mtn("revert", "testfile0"), 0, false, false)
 check(v1 == sha1("testfile0"))
-check(not exists("_MTN/work"))
+check(not qgrep("testfile0", "_MTN/revision"))
+check(not qgrep("foofile0", "_MTN/revision"))
 check(mtn("status"), 0, true)
 check(not qgrep("testfile0", "stdout"))
 check(not qgrep("foofile0", "stdout"))
