@@ -26,7 +26,7 @@ function getpathof(exe, ext)
   end
   local now = initial_dir.."/"..exe..ext
   if exists(now) then return gotit(now) end
-  for x in string.gfind(path, "[^"..char.."]*"..char) do
+  for x in string.gmatch(path, "[^"..char.."]*"..char) do
     local dir = string.sub(x, 0, -2)
     if string.find(dir, "[\\/]$") then
       dir = string.sub(dir, 0, -2)
@@ -46,6 +46,14 @@ monotone_path = getpathof("mtn")
 if monotone_path == nil then monotone_path = "mtn" end
 set_env("mtn", monotone_path)
 
+writefile_q("in", nil)
+prepare_redirect("in", "out", "err")
+execute(monotone_path, "--full-version")
+logfile:write(readfile_q("out"))
+unlogged_remove("in")
+unlogged_remove("out")
+unlogged_remove("err")
+
 -- NLS nuisances.
 for _,name in pairs({  "LANG",
                        "LANGUAGE",
@@ -120,9 +128,9 @@ function probe_node(filename, rsha, fsha)
 end
 
 function mtn_setup()
-  getstd("test_keys")
-  getstd("test_hooks.lua")
-  getstd("min_hooks.lua")
+  check(getstd("test_keys"))
+  check(getstd("test_hooks.lua"))
+  check(getstd("min_hooks.lua"))
   
   check(mtn("db", "init"), 0, false, false)
   check(mtn("read", "test_keys"), 0, false, false)
@@ -131,12 +139,17 @@ function mtn_setup()
 end
 
 function base_revision()
-  return (string.gsub(readfile("_MTN/revision"), "%s*$", ""))
+  local workrev = readfile("_MTN/revision")
+  local extract = string.gsub(workrev, "^.*old_revision %[(%x*)%].*$", "%1")
+  if extract == workrev then
+    err("failed to extract base revision from _MTN/revision")
+  end
+  return extract
 end
 
 function base_manifest()
   check(safe_mtn("automate", "get_manifest_of", base_revision()), 0, false)
-  copy("ts-stdout", "base_manifest_temp")
+  check(copy("ts-stdout", "base_manifest_temp"))
   return sha1("base_manifest_temp")
 end
 
@@ -365,7 +378,7 @@ table.insert(tests, "checkout_validates_target_directory")
 table.insert(tests, "checkout_creates_right__MTN_options")
 table.insert(tests, "trust_hooks_and_trusted_command")
 table.insert(tests, "attr_set_attr_get")
-table.insert(tests, "--rcfile_requires_extant_file")
+table.insert(tests, "_--rcfile_requires_extant_file")
 table.insert(tests, "persistent_netsync_server_-_revs_&_certs")
 table.insert(tests, "persistent_netsync_server_-_keys")
 table.insert(tests, "first_extent_normalization_pass")
@@ -387,7 +400,7 @@ table.insert(tests, "status_with_missing_files")
 table.insert(tests, "(imp)_persistent_netsync_server_-_keys_2")
 table.insert(tests, "update_1")
 table.insert(tests, "(todo)_vcheck")
-table.insert(tests, "--db_with_parent_dir")
+table.insert(tests, "_--db_with_parent_dir")
 table.insert(tests, "add_in_subdir")
 table.insert(tests, "(minor)_drop_in_subdir")
 table.insert(tests, "revert_in_subdirs")
@@ -480,7 +493,7 @@ table.insert(tests, "cvs_import_with_file_added_on_a_branch")
 table.insert(tests, "use_get_linesep_conv_hook")
 table.insert(tests, "add_workspace_commit_in_another")
 table.insert(tests, "update_to_non-existent_rev")
-table.insert(tests, "--author,_--date")
+table.insert(tests, "_--author,_--date")
 table.insert(tests, "update_does_not_stomp_non-monotone_files")
 table.insert(tests, "db_check_and_non-serious_errors")
 table.insert(tests, "db_kill_rev_locally_command")
@@ -494,7 +507,7 @@ table.insert(tests, "bad_packet_args")
 table.insert(tests, "commit_update_multiple_heads_message")
 table.insert(tests, "diffing_with_explicit_rev_same_as_wc_rev")
 table.insert(tests, "normalized_filenames")
-table.insert(tests, "workspace_inventory")
+table.insert(tests, "automate_inventory")
 table.insert(tests, "rename_file_to_dir")
 table.insert(tests, "replace_file_with_dir")
 table.insert(tests, "replace_dir_with_file")
@@ -513,7 +526,7 @@ table.insert(tests, "netsync_badhost_gives_nice_error")
 table.insert(tests, "checking_a_few_command_specific_options")
 table.insert(tests, "annotate_where_one_parent_is_full_copy")
 table.insert(tests, "cvs_import,_deleted_file_invariant")
-table.insert(tests, "--rcfile=-")
+table.insert(tests, "_--rcfile=-")
 table.insert(tests, "mtn_up")
 table.insert(tests, "merge((drop_a),_(rename_a_b,_patch_b))")
 table.insert(tests, "fail_cleanly_on_unreadable__MTN_options")
@@ -538,7 +551,7 @@ table.insert(tests, "merge_manual_file")
 table.insert(tests, "revert_works_with_restrictions")
 table.insert(tests, "status")
 table.insert(tests, "a_tricky_cvs_repository_with_tags")
-table.insert(tests, "--rcfile=directory")
+table.insert(tests, "_--rcfile=directory")
 table.insert(tests, "include()_and_includedir()_lua_functions")
 table.insert(tests, "lua_function_existsonpath")
 table.insert(tests, "db_kill_branch_certs_locally_command")
@@ -554,7 +567,7 @@ table.insert(tests, "b_and_t_selector_globbing")
 table.insert(tests, "diff_--external")
 table.insert(tests, "db_migrate_on_bad_schema")
 table.insert(tests, "list_branches")
-table.insert(tests, "database_check_for_normalization")
+table.insert(tests, "unnormalized_paths_in_database")
 table.insert(tests, "annotate_with_no_revs")
 table.insert(tests, "merging_(add_a,_rename_a_b)_with_(add_b)")
 table.insert(tests, "update_-b_foo_updates__MTN_options_correctly")
@@ -581,7 +594,7 @@ table.insert(tests, "diffing_a_file_within_revision_outside_a_workspace")
 table.insert(tests, "logging_a_file_within_revision_outside_a_workspace")
 table.insert(tests, "add_inside__MTN_")
 table.insert(tests, "annotate_file_whose_name_changed")
-table.insert(tests, "--confdir_option_and_get_confdir_lua_function_work")
+table.insert(tests, "_--confdir_option_and_get_confdir_lua_function_work")
 table.insert(tests, "database_is_closed_on_signal_exit")
 table.insert(tests, "update_-b_switches_branches_even_when_noop")
 table.insert(tests, "migrate_with_rosterify")
@@ -656,3 +669,20 @@ table.insert(tests, "diff_shows_renames")
 table.insert(tests, "dump_on_crash")
 table.insert(tests, "automate_tags")
 table.insert(tests, "restrictions_with_deletes")
+table.insert(tests, "log_with_restriction")
+table.insert(tests, "log_quits_on_SIGPIPE")
+table.insert(tests, "drop_directory_with_unversioned_files_and_merge")
+table.insert(tests, "checkout_-r_no_dir")
+table.insert(tests, "annotate_with_human_output")
+table.insert(tests, "automate_genkey")
+table.insert(tests, "migrate_workspace")
+table.insert(tests, "workspace_migration")
+table.insert(tests, "usage_output_streams")
+table.insert(tests, "automate_get_content_changed")
+table.insert(tests, "i18n_commit_messages")
+table.insert(tests, "ws_ops_with_wrong_node_type")
+table.insert(tests, "pivot_root_to_new_dir")
+table.insert(tests, "multiple_message_commit")
+table.insert(tests, "netsync_epoch_refinement")
+table.insert(tests, "disapproving_with_message")
+table.insert(tests, "add_ignored")
