@@ -147,6 +147,82 @@ pid_t process_spawn(const char * const argv[])
   return (pid_t)pi.hProcess;
 }
 
+struct redir
+{
+  struct bad_redir {};
+  HANDLE saved;
+  int what;
+  redir(int which, char const * file);
+  ~redir();
+};
+redir::redir(int which, char const * filename)
+ : what(which)
+{
+  HANDLE file;
+  SECURITY_ATTRIBUTES sa;
+  sa.nLength = sizeof(SECURITY_ATTRIBUTES);
+  sa.lpSecurityDescriptor = 0;
+  sa.bInheritHandle = true;
+  
+  file = CreateFile(filename,
+                    (which==0?GENERIC_READ:GENERIC_WRITE),
+                    FILE_SHARE_READ,
+                    &sa,
+                    (which==0?OPEN_EXISTING:CREATE_ALWAYS),
+                    FILE_ATTRIBUTE_NORMAL,
+                    NULL);
+  switch(which)
+  {
+  case 0:
+    saved = GetStdHandle(STD_INPUT_HANDLE);
+    SetStdHandle(STD_INPUT_HANDLE, file);
+    break;
+  case 1:
+    saved = GetStdHandle(STD_OUTPUT_HANDLE);
+    SetStdHandle(STD_OUTPUT_HANDLE, file);
+    break;
+  case 2:
+    saved = GetStdHandle(STD_ERROR_HANDLE);
+    SetStdHandle(STD_ERROR_HANDLE, file);
+    break;
+  }
+}
+redir::~redir()
+{
+  switch(what)
+  {
+  case 0:
+    CloseHandle(GetStdHandle(STD_INPUT_HANDLE));
+    SetStdHandle(STD_INPUT_HANDLE, saved);
+    break;
+  case 1:
+    CloseHandle(GetStdHandle(STD_OUTPUT_HANDLE));
+    SetStdHandle(STD_OUTPUT_HANDLE, saved);
+    break;
+  case 2:
+    CloseHandle(GetStdHandle(STD_ERROR_HANDLE));
+    SetStdHandle(STD_ERROR_HANDLE, saved);
+    break;
+  }
+}
+pid_t process_spawn_redirected(char const * in,
+                               char const * out,
+                               char const * err,
+                               char const * const argv[])
+{
+  try
+    {
+      redir i(0, in);
+      redir o(1, out);
+      redir e(2, err);
+      return process_spawn(argv);
+    }
+  catch (redir::bad_redir)
+    {
+      return -1;
+    }
+}
+
 int process_wait(pid_t pid, int *res, int timeout)
 {
   HANDLE hProcess = (HANDLE)pid;
